ID_BASE_EXCEPTIONS = set(
    """
aba-aba
abah-abah
abal-abal
abang-abang
abar-abar
abong-abong
abrit-abrit
abrit-abritan
abu-abu
abuh-abuhan
abuk-abuk
abun-abun
acak-acak
acak-acakan
acang-acang
acap-acap
aci-aci
aci-acian
aci-acinya
aco-acoan
ad-blocker
ad-interim
ada-ada
ada-adanya
ada-adanyakah
adang-adang
adap-adapan
add-on
add-ons
adik-adik
adik-beradik
aduk-adukan
after-sales
agak-agak
agak-agih
agama-agama
agar-agar
age-related
agut-agut
air-air
air-cooled
air-to-air
ajak-ajak
ajar-ajar
aji-aji
akal-akal
akal-akalan
akan-akan
akar-akar
akar-akaran
akhir-akhir
akhir-akhirnya
aki-aki
aksi-aksi
alah-mengalahi
alai-belai
alan-alan
alang-alang
alang-alangan
alap-alap
alat-alat
ali-ali
alif-alifan
alih-alih
aling-aling
aling-alingan
alip-alipan
all-electric
all-in-one
all-out
all-time
alon-alon
alt-right
alt-text
alu-alu
alu-aluan
alun-alun
alur-alur
alur-aluran
always-on
amai-amai
amatir-amatiran
ambah-ambah
ambai-ambai
ambil-mengambil
ambreng-ambrengan
ambring-ambringan
ambu-ambu
ambung-ambung
amin-amin
amit-amit
ampai-ampai
amprung-amprungan
amung-amung
anai-anai
anak-anak
anak-anakan
anak-beranak
anak-cucu
anak-istri
ancak-ancak
ancang-ancang
ancar-ancar
andang-andang
andeng-andeng
aneh-aneh
angan-angan
anggar-anggar
anggaran-red
anggota-anggota
anggung-anggip
angin-angin
angin-anginan
angkal-angkal
angkul-angkul
angkup-angkup
angkut-angkut
ani-ani
aning-aning
anjang-anjang
anjing-anjing
anjung-anjung
anjung-anjungan
antah-berantah
antar-antar
antar-mengantar
ante-mortem
antek-antek
anter-anter
antihuru-hara
anting-anting
antung-antung
anyam-menganyam
anyang-anyang
apa-apa
apa-apaan
apel-apel
api-api
apit-apit
aplikasi-aplikasi
apotek-apotek
aprit-apritan
apu-apu
apung-apung
arah-arah
arak-arak
arak-arakan
aram-aram
arek-arek
arem-arem
ari-ari
artis-artis
aru-aru
arung-arungan
asa-asaan
asal-asalan
asal-muasal
asal-usul
asam-asaman
asas-asas
aset-aset
asmaul-husna
asosiasi-asosiasi
asuh-asuh
asyik-asyiknya
atas-mengatasi
ati-ati
atung-atung
aturan-aturan
audio-video
audio-visual
auto-brightness
auto-complete
auto-focus
auto-play
auto-update
avant-garde
awan-awan
awan-berawan
awang-awang
awang-gemawang
awar-awar
awat-awat
awik-awik
awut-awutan
ayah-anak
ayak-ayak
ayam-ayam
ayam-ayaman
ayang-ayang
ayat-ayat
ayeng-ayengan
ayun-temayun
ayut-ayutan
ba-bi-bu
back-to-back
back-up
badan-badan
bade-bade
badut-badut
bagi-bagi
bahan-bahan
bahu-membahu
baik-baik
bail-out
bajang-bajang
baji-baji
balai-balai
balam-balam
balas-berbalas
balas-membalas
bale-bale
baling-baling
ball-playing
balon-balon
balut-balut
band-band
bandara-bandara
bangsa-bangsa
bangun-bangun
bangunan-bangunan
bank-bank
bantah-bantah
bantahan-bantahan
bantal-bantal
banyak-banyak
bapak-anak
bapak-bapak
bapak-ibu
bapak-ibunya
barang-barang
barat-barat
barat-daya
barat-laut
barau-barau
bare-bare
bareng-bareng
bari-bari
barik-barik
baris-berbaris
baru-baru
baru-batu
barung-barung
basa-basi
bata-bata
batalyon-batalyon
batang-batang
batas-batas
batir-batir
batu-batu
batuk-batuk
batung-batung
bau-bauan
bawa-bawa
bayan-bayan
bayang-bayang
bayi-bayi
bea-cukai
bedeng-bedeng
bedil-bedal
bedil-bedilan
begana-begini
bek-bek
bekal-bekalan
bekerdom-kerdom
bekertak-kertak
belang-belang
belat-belit
beliau-beliau
belu-belai
belum-belum
benar-benar
benda-benda
bengang-bengut
benggal-benggil
bengkal-bengkil
bengkang-bengkok
bengkang-bengkong
bengkang-bengkung
benteng-benteng
bentuk-bentuk
benua-benua
ber-selfie
berabad-abad
berabun-rabun
beracah-acah
berada-ada
beradik-berkakak
beragah-agah
beragak-agak
beragam-ragam
beraja-raja
berakit-rakit
beraku-akuan
beralu-aluan
beralun-alun
beramah-ramah
beramah-ramahan
beramah-tamah
beramai-ramai
berambai-ambai
berambal-ambalan
berambil-ambil
beramuk-amuk
beramuk-amukan
berandai-andai
berandai-randai
beraneh-aneh
berang-berang
berangan-angan
beranggap-anggapan
berangguk-angguk
berangin-angin
berangka-angka
berangka-angkaan
berangkai-rangkai
berangkap-rangkapan
berani-berani
beranja-anja
berantai-rantai
berapi-api
berapung-apung
berarak-arakan
beras-beras
berasak-asak
berasak-asakan
berasap-asap
berasing-asingan
beratus-ratus
berawa-rawa
berawas-awas
berayal-ayalan
berayun-ayun
berbagai-bagai
berbahas-bahasan
berbahasa-bahasa
berbaik-baikan
berbait-bait
berbala-bala
berbalas-balasan
berbalik-balik
berbalun-balun
berbanjar-banjar
berbantah-bantah
berbanyak-banyak
berbarik-barik
berbasa-basi
berbasah-basah
berbatu-batu
berbayang-bayang
berbecak-becak
berbeda-beda
berbedil-bedilan
berbega-bega
berbeka-beka
berbelah-belah
berbelakang-belakangan
berbelang-belang
berbelau-belauan
berbeli-beli
berbeli-belian
berbelit-belit
berbelok-belok
berbenang-benang
berbenar-benar
berbencah-bencah
berbencol-bencol
berbenggil-benggil
berbentol-bentol
berbentong-bentong
berberani-berani
berbesar-besar
berbidai-bidai
berbiduk-biduk
berbiku-biku
berbilik-bilik
berbinar-binar
berbincang-bincang
berbingkah-bingkah
berbintang-bintang
berbintik-bintik
berbintil-bintil
berbisik-bisik
berbolak-balik
berbolong-bolong
berbondong-bondong
berbongkah-bongkah
berbuai-buai
berbual-bual
berbudak-budak
berbukit-bukit
berbulan-bulan
berbunga-bunga
berbuntut-buntut
berbunuh-bunuhan
berburu-buru
berburuk-buruk
berbutir-butir
bercabang-cabang
bercaci-cacian
bercakap-cakap
bercakar-cakaran
bercamping-camping
bercantik-cantik
bercari-cari
bercari-carian
bercarik-carik
bercarut-carut
bercebar-cebur
bercepat-cepat
bercerai-berai
bercerai-cerai
bercetai-cetai
berciap-ciap
bercikun-cikun
bercinta-cintaan
bercita-cita
berciut-ciut
bercompang-camping
berconteng-conteng
bercoreng-coreng
bercoreng-moreng
bercuang-caing
bercuit-cuit
bercumbu-cumbu
bercumbu-cumbuan
bercura-bura
bercura-cura
berdada-dadaan
berdahulu-dahuluan
berdalam-dalam
berdalih-dalih
berdampung-dampung
berdebar-debar
berdecak-decak
berdecap-decap
berdecup-decup
berdecut-decut
berdedai-dedai
berdegap-degap
berdegar-degar
berdeham-deham
berdekah-dekah
berdekak-dekak
berdekap-dekapan
berdekat-dekat
berdelat-delat
berdembai-dembai
berdembun-dembun
berdempang-dempang
berdempet-dempet
berdencing-dencing
berdendam-dendaman
berdengkang-dengkang
berdengut-dengut
berdentang-dentang
berdentum-dentum
berdentung-dentung
berdenyar-denyar
berdenyut-denyut
berdepak-depak
berdepan-depan
berderai-derai
berderak-derak
berderam-deram
berderau-derau
berderik-derik
berdering-dering
berderung-derung
berderus-derus
berdesak-desakan
berdesik-desik
berdesing-desing
berdesus-desus
berdikit-dikit
berdingkit-dingkit
berdua-dua
berduri-duri
berduru-duru
berduyun-duyun
berebut-rebut
berebut-rebutan
beregang-regang
berek-berek
berembut-rembut
berempat-empat
berenak-enak
berencel-encel
bereng-bereng
berenggan-enggan
berenteng-renteng
beresa-esaan
beresah-resah
berfoya-foya
bergagah-gagahan
bergagap-gagap
bergagau-gagau
bergalur-galur
berganda-ganda
berganjur-ganjur
berganti-ganti
bergarah-garah
bergaruk-garuk
bergaya-gaya
bergegas-gegas
bergelang-gelang
bergelap-gelap
bergelas-gelasan
bergeleng-geleng
bergemal-gemal
bergembar-gembor
bergembut-gembut
bergepok-gepok
bergerek-gerek
bergesa-gesa
bergilir-gilir
bergolak-golak
bergolek-golek
bergolong-golong
bergores-gores
bergotong-royong
bergoyang-goyang
bergugus-gugus
bergulung-gulung
bergulut-gulut
bergumpal-gumpal
bergunduk-gunduk
bergunung-gunung
berhadap-hadapan
berhamun-hamun
berhandai-handai
berhanyut-hanyut
berhari-hari
berhati-hati
berhati-hatilah
berhektare-hektare
berhilau-hilau
berhormat-hormat
berhujan-hujan
berhura-hura
beri-beri
beri-memberi
beria-ia
beria-ria
beriak-riak
beriba-iba
beribu-ribu
berigi-rigi
berimpit-impit
berindap-indap
bering-bering
beringat-ingat
beringgit-ringgit
berintik-rintik
beriring-iring
beriring-iringan
berita-berita
berjabir-jabir
berjaga-jaga
berjagung-jagung
berjalan-jalan
berjalar-jalar
berjalin-jalin
berjalur-jalur
berjam-jam
berjari-jari
berjauh-jauhan
berjegal-jegalan
berjejal-jejal
berjela-jela
berjengkek-jengkek
berjenis-jenis
berjenjang-jenjang
berjilid-jilid
berjinak-jinak
berjingkat-jingkat
berjingkik-jingkik
berjingkrak-jingkrak
berjongkok-jongkok
berjubel-jubel
berjujut-jujutan
berjulai-julai
berjumbai-jumbai
berjumbul-jumbul
berjuntai-juntai
berjurai-jurai
berjurus-jurus
berjuta-juta
berka-li-kali
berkabu-kabu
berkaca-kaca
berkaing-kaing
berkait-kaitan
berkala-kala
berkali-kali
berkamit-kamit
berkanjar-kanjar
berkaok-kaok
berkarung-karung
berkasak-kusuk
berkasih-kasihan
berkata-kata
berkatak-katak
berkecai-kecai
berkecek-kecek
berkecil-kecil
berkecil-kecilan
berkedip-kedip
berkejang-kejang
berkejap-kejap
berkejar-kejaran
berkelar-kelar
berkelepai-kelepai
berkelip-kelip
berkelit-kelit
berkelok-kelok
berkelompok-kelompok
berkelun-kelun
berkembur-kembur
berkempul-kempul
berkena-kenaan
berkenal-kenalan
berkendur-kendur
berkeok-keok
berkepak-kepak
berkepal-kepal
berkeping-keping
berkepul-kepul
berkeras-kerasan
berkering-kering
berkeritik-keritik
berkeruit-keruit
berkerut-kerut
berketai-ketai
berketak-ketak
berketak-ketik
berketap-ketap
berketap-ketip
berketar-ketar
berketi-keti
berketil-ketil
berketuk-ketak
berketul-ketul
berkial-kial
berkian-kian
berkias-kias
berkias-kiasan
berkibar-kibar
berkilah-kilah
berkilap-kilap
berkilat-kilat
berkilau-kilauan
berkilo-kilo
berkimbang-kimbang
berkinja-kinja
berkipas-kipas
berkira-kira
berkirim-kiriman
berkisar-kisar
berkoak-koak
berkoar-koar
berkobar-kobar
berkobok-kobok
berkocak-kocak
berkodi-kodi
berkolek-kolek
berkomat-kamit
berkopah-kopah
berkoper-koper
berkotak-kotak
berkuat-kuat
berkuat-kuatan
berkumur-kumur
berkunang-kunang
berkunar-kunar
berkunjung-kunjungan
berkurik-kurik
berkurun-kurun
berkusau-kusau
berkusu-kusu
berkusut-kusut
berkuting-kuting
berkutu-kutuan
berlabun-labun
berlain-lainan
berlaju-laju
berlalai-lalai
berlama-lama
berlambai-lambai
berlambak-lambak
berlampang-lampang
berlanggar-langgar
berlapang-lapang
berlapis-lapis
berlapuk-lapuk
berlarah-larah
berlarat-larat
berlari-lari
berlari-larian
berlarih-larih
berlarik-larik
berlarut-larut
berlawak-lawak
berlayap-layapan
berlebih-lebih
berlebih-lebihan
berleha-leha
berlekas-lekas
berlekas-lekasan
berlekat-lekat
berlekuk-lekuk
berlempar-lemparan
berlena-lena
berlengah-lengah
berlenggak-lenggok
berlenggek-lenggek
berlenggok-lenggok
berleret-leret
berletih-letih
berliang-liuk
berlibat-libat
berligar-ligar
berliku-liku
berlikur-likur
berlimbak-limbak
berlimpah-limpah
berlimpap-limpap
berlimpit-limpit
berlinang-linang
berlindak-lindak
berlipat-lipat
berlomba-lomba
berlompok-lompok
berloncat-loncatan
berlopak-lopak
berlubang-lubang
berlusin-lusin
bermaaf-maafan
bermabuk-mabukan
bermacam-macam
bermain-main
bermalam-malam
bermalas-malas
bermalas-malasan
bermanik-manik
bermanis-manis
bermanja-manja
bermasak-masak
bermati-mati
bermegah-megah
bermemek-memek
bermenung-menung
bermesra-mesraan
bermewah-mewah
bermewah-mewahan
berminggu-minggu
berminta-minta
berminyak-minyak
bermuda-muda
bermudah-mudah
bermuka-muka
bermula-mula
bermuluk-muluk
bermulut-mulut
bernafsi-nafsi
bernaka-naka
bernala-nala
bernanti-nanti
berniat-niat
bernyala-nyala
berogak-ogak
beroleng-oleng
berolok-olok
beromong-omong
beroncet-roncet
beronggok-onggok
berorang-orang
beroyal-royal
berpada-pada
berpadu-padu
berpahit-pahit
berpair-pair
berpal-pal
berpalu-palu
berpalu-paluan
berpalun-palun
berpanas-panas
berpandai-pandai
berpandang-pandangan
berpangkat-pangkat
berpanjang-panjang
berpantun-pantun
berpasang-pasang
berpasang-pasangan
berpasuk-pasuk
berpayah-payah
berpeluh-peluh
berpeluk-pelukan
berpenat-penat
berpencar-pencar
berpendar-pendar
berpenggal-penggal
berperai-perai
berperang-perangan
berpesai-pesai
berpesta-pesta
berpesuk-pesuk
berpetak-petak
berpeti-peti
berpihak-pihak
berpijar-pijar
berpikir-pikir
berpikul-pikul
berpilih-pilih
berpilin-pilin
berpindah-pindah
berpintal-pintal
berpirau-pirau
berpisah-pisah
berpolah-polah
berpolok-polok
berpongah-pongah
berpontang-panting
berporah-porah
berpotong-potong
berpotong-potongan
berpuak-puak
berpual-pual
berpugak-pugak
berpuing-puing
berpukas-pukas
berpuluh-puluh
berpulun-pulun
berpuntal-puntal
berpura-pura
berpusar-pusar
berpusing-pusing
berpusu-pusu
berputar-putar
berrumpun-rumpun
bersaf-saf
bersahut-sahutan
bersakit-sakit
bersalah-salahan
bersalam-salaman
bersalin-salin
bersalip-salipan
bersama-sama
bersambar-sambaran
bersambut-sambutan
bersampan-sampan
bersantai-santai
bersapa-sapaan
bersarang-sarang
bersedan-sedan
bersedia-sedia
bersedu-sedu
bersejuk-sejuk
bersekat-sekat
berselang-selang
berselang-seli
berselang-seling
berselang-tenggang
berselit-selit
berseluk-beluk
bersembunyi-sembunyi
bersembunyi-sembunyian
bersembur-semburan
bersempit-sempit
bersenang-senang
bersenang-senangkan
bersenda-senda
bersendi-sendi
bersenggang-senggang
bersenggau-senggau
bersepah-sepah
bersepak-sepakan
bersepi-sepi
berserak-serak
berseri-seri
berseru-seru
bersesak-sesak
bersetai-setai
bersia-sia
bersiap-siap
bersiar-siar
bersih-bersih
bersikut-sikutan
bersilir-silir
bersimbur-simburan
bersinau-sinau
bersopan-sopan
bersorak-sorai
bersuap-suapan
bersudah-sudah
bersuka-suka
bersuka-sukaan
bersuku-suku
bersulang-sulang
bersumpah-sumpahan
bersungguh-sungguh
bersungut-sungut
bersunyi-sunyi
bersuruk-surukan
bersusah-susah
bersusuk-susuk
bersusuk-susukan
bersutan-sutan
bertabur-tabur
bertahan-tahan
bertahu-tahu
bertahun-tahun
bertajuk-tajuk
bertakik-takik
bertala-tala
bertalah-talah
bertali-tali
bertalu-talu
bertalun-talun
bertambah-tambah
bertanda-tandaan
bertangis-tangisan
bertangkil-tangkil
bertanya-tanya
bertarik-tarikan
bertatai-tatai
bertatap-tatapan
bertatih-tatih
bertawan-tawan
bertawar-tawaran
bertebu-tebu
bertebu-tebukan
berteguh-teguh
berteguh-teguhan
berteka-teki
bertelang-telang
bertelau-telau
bertele-tele
bertembuk-tembuk
bertempat-tempat
bertempuh-tempuh
bertenang-tenang
bertenggang-tenggangan
bertentu-tentu
bertepek-tepek
berterang-terang
berterang-terangan
berteriak-teriak
bertikam-tikaman
bertimbal-timbalan
bertimbun-timbun
bertimpa-timpa
bertimpas-timpas
bertingkah-tingkah
bertingkat-tingkat
bertinjau-tinjauan
bertiras-tiras
bertitar-titar
bertitik-titik
bertoboh-toboh
bertolak-tolak
bertolak-tolakan
bertolong-tolongan
bertonjol-tonjol
bertruk-truk
bertua-tua
bertua-tuaan
bertual-tual
bertubi-tubi
bertukar-tukar
bertukar-tukaran
bertukas-tukas
bertumpak-tumpak
bertumpang-tindih
bertumpuk-tumpuk
bertunda-tunda
bertunjuk-tunjukan
bertura-tura
berturut-turut
bertutur-tutur
beruas-ruas
berubah-ubah
berulang-alik
berulang-ulang
berumbai-rumbai
berundak-undak
berundan-undan
berundung-undung
berunggas-runggas
berunggun-unggun
berunggut-unggut
berungkur-ungkuran
beruntai-untai
beruntun-runtun
beruntung-untung
berunyai-unyai
berupa-rupa
berura-ura
beruris-uris
berurut-urutan
berwarna-warna
berwarna-warni
berwindu-windu
berwiru-wiru
beryang-yang
besar-besar
besar-besaran
betak-betak
beti-beti
betik-betik
betul-betul
biang-biang
biar-biar
biaya-biaya
bicu-bicu
bidadari-bidadari
bidang-bidang
bijak-bijaklah
biji-bijian
bila-bila
bilang-bilang
bincang-bincang
bincang-bincut
bingkah-bingkah
bini-binian
bintang-bintang
bintik-bintik
bio-oil
biri-biri
biru-biru
biru-hitam
biru-kuning
bisik-bisik
biti-biti
blak-blakan
blok-blok
bocah-bocah
bohong-bohong
bohong-bohongan
bola-bola
bolak-balik
bolang-baling
boleh-boleh
bom-bom
bomber-bomber
bonek-bonek
bongkar-bangkir
bongkar-membongkar
bongkar-pasang
boro-boro
bos-bos
bottom-up
box-to-box
boyo-boyo
buah-buahan
buang-buang
buat-buatan
buaya-buaya
bubun-bubun
bugi-bugi
build-up
built-in
built-up
buka-buka
buka-bukaan
buka-tutup
bukan-bukan
bukti-bukti
buku-buku
bulan-bulan
bulan-bulanan
bulang-baling
bulang-bulang
bulat-bulat
buli-buli
bulu-bulu
buluh-buluh
bulus-bulus
bunga-bunga
bunga-bungaan
bunuh-membunuh
bunyi-bunyian
bupati-bupati
bupati-wakil
buru-buru
burung-burung
burung-burungan
bus-bus
business-to-business
busur-busur
butir-butir
by-pass
bye-bye
cabang-cabang
cabik-cabik
cabik-mencabik
cabup-cawabup
caci-maki
cagub-cawagub
caing-caing
cakar-mencakar
cakup-mencakup
calak-calak
calar-balar
caleg-caleg
calo-calo
calon-calon
campang-camping
campur-campur
capres-cawapres
cara-cara
cari-cari
cari-carian
carut-marut
catch-up
cawali-cawawali
cawe-cawe
cawi-cawi
cebar-cebur
celah-celah
celam-celum
celangak-celinguk
celas-celus
celedang-celedok
celengkak-celengkok
celingak-celinguk
celung-celung
cemas-cemas
cenal-cenil
cengar-cengir
cengir-cengir
cengis-cengis
cengking-mengking
centang-perenang
cepat-cepat
ceplas-ceplos
cerai-berai
cerita-cerita
ceruk-menceruk
ceruk-meruk
cetak-biru
cetak-mencetak
cetar-ceter
check-in
check-ins
check-up
chit-chat
choki-choki
cingak-cinguk
cipika-cipiki
ciri-ciri
ciri-cirinya
cirit-birit
cita-cita
cita-citaku
close-up
closed-circuit
coba-coba
cobak-cabik
cobar-cabir
cola-cala
colang-caling
comat-comot
comot-comot
compang-camping
computer-aided
computer-generated
condong-mondong
congak-cangit
conggah-canggih
congkah-cangkih
congkah-mangkih
copak-capik
copy-paste
corak-carik
corat-coret
coreng-moreng
coret-coret
crat-crit
cross-border
cross-dressing
crypto-ransomware
cuang-caing
cublak-cublak
cubung-cubung
culik-culik
cuma-cuma
cumi-cumi
cungap-cangip
cupu-cupu
dabu-dabu
daerah-daerah
dag-dag
dag-dig-dug
daging-dagingan
dahulu-mendahului
dalam-dalam
dali-dali
dam-dam
danau-danau
dansa-dansi
dapil-dapil
dapur-dapur
dari-dari
daru-daru
dasar-dasar
datang-datang
datang-mendatangi
daun-daun
daun-daunan
dawai-dawai
dayang-dayang
dayung-mayung
debak-debuk
debu-debu
deca-core
decision-making
deep-lying
deg-degan
degap-degap
dekak-dekak
dekat-dekat
dengar-dengaran
dengking-mendengking
departemen-departemen
depo-depo
deputi-deputi
desa-desa
desa-kota
desas-desus
detik-detik
dewa-dewa
dewa-dewi
dewan-dewan
dewi-dewi
dial-up
diam-diam
dibayang-bayangi
dibuat-buat
diiming-imingi
dilebih-lebihkan
dimana-mana
dimata-matai
dinas-dinas
dinul-Islam
diobok-obok
diolok-olok
direksi-direksi
direktorat-direktorat
dirjen-dirjen
dirut-dirut
ditunggu-tunggu
divisi-divisi
do-it-yourself
doa-doa
dog-dog
doggy-style
dokok-dokok
dolak-dalik
dor-doran
dorong-mendorong
dosa-dosa
dress-up
drive-in
dua-dua
dua-duaan
dua-duanya
dubes-dubes
duduk-duduk
dugaan-dugaan
dulang-dulang
duri-duri
duta-duta
dwi-kewarganegaraan
e-arena
e-billing
e-budgeting
e-cctv
e-class
e-commerce
e-counting
e-elektronik
e-entertainment
e-evolution
e-faktur
e-filing
e-fin
e-form
e-government
e-govt
e-hakcipta
e-id
e-info
e-katalog
e-ktp
e-leadership
e-lhkpn
e-library
e-loket
e-m1
e-money
e-news
e-nisn
e-npwp
e-paspor
e-paten
e-pay
e-perda
e-perizinan
e-planning
e-polisi
e-power
e-punten
e-retribusi
e-samsat
e-sport
e-store
e-tax
e-ticketing
e-tilang
e-toll
e-visa
e-voting
e-wallet
e-warong
ecek-ecek
eco-friendly
eco-park
edan-edanan
editor-editor
editor-in-chief
efek-efek
ekonomi-ekonomi
eksekutif-legislatif
ekspor-impor
elang-elang
elemen-elemen
emak-emak
embuh-embuhan
empat-empat
empek-empek
empet-empetan
empok-empok
empot-empotan
enak-enak
encal-encal
end-to-end
end-user
endap-endap
endut-endut
endut-endutan
engah-engah
engap-engap
enggan-enggan
engkah-engkah
engket-engket
entah-berentah
enten-enten
entry-level
equity-linked
erang-erot
erat-erat
erek-erek
ereng-ereng
erong-erong
esek-esek
ex-officio
exchange-traded
exercise-induced
extra-time
face-down
face-to-face
fair-play
fakta-fakta
faktor-faktor
fakultas-fakultas
fase-fase
fast-food
feed-in
fifty-fifty
file-file
first-leg
first-team
fitur-fitur
fitur-fiturnya
fixed-income
flip-flop
flip-plop
fly-in
follow-up
foto-foto
foya-foya
fraksi-fraksi
free-to-play
front-end
fungsi-fungsi
gaba-gaba
gabai-gabai
gada-gada
gading-gading
gadis-gadis
gado-gado
gail-gail
gajah-gajah
gajah-gajahan
gala-gala
galeri-galeri
gali-gali
gali-galian
galing-galing
galu-galu
gamak-gamak
gambar-gambar
gambar-menggambar
gamit-gamitan
gampang-gampangan
gana-gini
ganal-ganal
ganda-berganda
ganjal-mengganjal
ganjil-genap
ganteng-ganteng
gantung-gantung
gapah-gopoh
gara-gara
garah-garah
garis-garis
gasak-gasakan
gatal-gatal
gaun-gaun
gawar-gawar
gaya-gayanya
gayang-gayang
ge-er
gebyah-uyah
gebyar-gebyar
gedana-gedini
gedebak-gedebuk
gedebar-gedebur
gedung-gedung
gelang-gelang
gelap-gelapan
gelar-gelar
gelas-gelas
gelembung-gelembungan
geleng-geleng
geli-geli
geliang-geliut
geliat-geliut
gembar-gembor
gembrang-gembreng
gempul-gempul
gempur-menggempur
gendang-gendang
gengsi-gengsian
genjang-genjot
genjot-genjotan
genjrang-genjreng
genome-wide
geo-politik
gerabak-gerubuk
gerak-gerik
gerak-geriknya
gerakan-gerakan
gerbas-gerbus
gereja-gereja
gereng-gereng
geriak-geriuk
gerit-gerit
gerot-gerot
geruh-gerah
getak-getuk
getem-getem
geti-geti
gial-gial
gial-giul
gila-gila
gila-gilaan
gilang-gemilang
gilap-gemilap
gili-gili
giling-giling
gilir-bergilir
ginang-ginang
girap-girap
girik-girik
giring-giring
go-auto
go-bills
go-bluebird
go-box
go-car
go-clean
go-food
go-glam
go-jek
go-kart
go-mart
go-massage
go-med
go-points
go-pulsa
go-ride
go-send
go-shop
go-tix
go-to-market
goak-goak
goal-line
gol-gol
golak-galik
gondas-gandes
gonjang-ganjing
gonjlang-ganjling
gonta-ganti
gontok-gontokan
gorap-gorap
gorong-gorong
gotong-royong
gresek-gresek
gua-gua
gual-gail
gubernur-gubernur
gudu-gudu
gula-gula
gulang-gulang
gulung-menggulung
guna-ganah
guna-guna
gundala-gundala
guntang-guntang
gunung-ganang
gunung-gemunung
gunung-gunungan
guru-guru
habis-habis
habis-habisan
hak-hak
hak-hal
hakim-hakim
hal-hal
halai-balai
half-time
hama-hama
hampir-hampir
hancur-hancuran
hancur-menghancurkan
hands-free
hands-on
hang-out
hantu-hantu
happy-happy
harap-harap
harap-harapan
hard-disk
harga-harga
hari-hari
harimau-harimau
harum-haruman
hasil-hasil
hasta-wara
hat-trick
hati-hati
hati-hatilah
head-mounted
head-to-head
head-up
heads-up
heavy-duty
hebat-hebatan
hewan-hewan
hexa-core
hidup-hidup
hidup-mati
hila-hila
hilang-hilang
hina-menghinakan
hip-hop
hiru-biru
hiru-hara
hiruk-pikuk
hitam-putih
hitung-hitung
hitung-hitungan
hormat-menghormati
hot-swappable
hotel-hotel
how-to
hubar-habir
hubaya-hubaya
hukum-red
hukuman-hukuman
hula-hoop
hula-hula
hulu-hilir
humas-humas
hura-hura
huru-hara
ibar-ibar
ibu-anak
ibu-ibu
icak-icak
icip-icip
idam-idam
ide-ide
igau-igauan
ikan-ikan
ikut-ikut
ikut-ikutan
ilam-ilam
ilat-ilatan
ilmu-ilmu
imbang-imbangan
iming-iming
imut-imut
inang-inang
inca-binca
incang-incut
industri-industri
ingar-bingar
ingar-ingar
ingat-ingat
ingat-ingatan
ingau-ingauan
inggang-inggung
injak-injak
input-output
instansi-instansi
instant-on
instrumen-instrumen
inter-governmental
ira-ira
irah-irahan
iras-iras
iring-iringan
iris-irisan
isak-isak
isat-bb
iseng-iseng
istana-istana
istri-istri
isu-isu
iya-iya
jabatan-jabatan
jadi-jadian
jagoan-jagoan
jaja-jajaan
jaksa-jaksa
jala-jala
jalan-jalan
jali-jali
jalin-berjalin
jalin-menjalin
jam-jam
jamah-jamahan
jambak-jambakan
jambu-jambu
jampi-jampi
janda-janda
jangan-jangan
janji-janji
jarang-jarang
jari-jari
jaring-jaring
jarum-jarum
jasa-jasa
jatuh-bangun
jauh-dekat
jauh-jauh
jawi-jawi
jebar-jebur
jebat-jebatan
jegal-jegalan
jejak-jejak
jelang-menjelang
jelas-jelas
jelur-jelir
jembatan-jembatan
jenazah-jenazah
jendal-jendul
jenderal-jenderal
jenggar-jenggur
jenis-jenis
jenis-jenisnya
jentik-jentik
jerah-jerih
jinak-jinak
jiwa-jiwa
joli-joli
jolong-jolong
jongkang-jangking
jongkar-jangkir
jongkat-jangkit
jor-joran
jotos-jotosan
juak-juak
jual-beli
juang-juang
julo-julo
julung-julung
julur-julur
jumbai-jumbai
jungkang-jungkit
jungkat-jungkit
jurai-jurai
kabang-kabang
kabar-kabari
kabir-kabiran
kabruk-kabrukan
kabu-kabu
kabupaten-kabupaten
kabupaten-kota
kaca-kaca
kacang-kacang
kacang-kacangan
kacau-balau
kadang-kadang
kader-kader
kades-kades
kadis-kadis
kail-kail
kain-kain
kait-kait
kakak-adik
kakak-beradik
kakak-kakak
kakek-kakek
kakek-nenek
kaki-kaki
kala-kala
kalau-kalau
kaleng-kalengan
kali-kalian
kalimat-kalimat
kalung-kalung
kalut-malut
kambing-kambing
kamit-kamit
kampung-kampung
kampus-kampus
kanak-kanak
kanak-kanan
kanan-kanak
kanan-kiri
kangen-kangenan
kanwil-kanwil
kapa-kapa
kapal-kapal
kapan-kapan
kapolda-kapolda
kapolres-kapolres
kapolsek-kapolsek
kapu-kapu
karang-karangan
karang-mengarang
kareseh-peseh
karut-marut
karya-karya
kasak-kusuk
kasus-kasus
kata-kata
katang-katang
kava-kava
kawa-kawa
kawan-kawan
kawin-cerai
kawin-mawin
kayu-kayu
kayu-kayuan
ke-Allah-an
keabu-abuan
kearab-araban
keasyik-asyikan
kebarat-baratan
kebasah-basahan
kebat-kebit
kebata-bataan
kebayi-bayian
kebelanda-belandaan
keberlarut-larutan
kebesar-hatian
kebiasaan-kebiasaan
kebijakan-kebijakan
kebiru-biruan
kebudak-budakan
kebun-kebun
kebut-kebutan
kecamatan-kecamatan
kecentang-perenangan
kecil-kecil
kecil-kecilan
kecil-mengecil
kecokelat-cokelatan
kecomak-kecimik
kecuh-kecah
kedek-kedek
kedekak-kedekik
kedesa-desaan
kedubes-kedubes
kedutaan-kedutaan
keempat-empatnya
kegadis-gadisan
kegelap-gelapan
kegiatan-kegiatan
kegila-gilaan
kegirang-girangan
kehati-hatian
keheran-heranan
kehijau-hijauan
kehitam-hitaman
keinggris-inggrisan
kejaga-jagaan
kejahatan-kejahatan
kejang-kejang
kejar-kejar
kejar-kejaran
kejar-mengejar
kejingga-jinggaan
kejut-kejut
kejutan-kejutan
kekabur-kaburan
kekanak-kanakan
kekoboi-koboian
kekota-kotaan
kekuasaan-kekuasaan
kekuning-kuningan
kelak-kelik
kelak-keluk
kelaki-lakian
kelang-kelok
kelap-kelip
kelasah-kelusuh
kelek-kelek
kelek-kelekan
kelemak-kelemek
kelik-kelik
kelip-kelip
kelompok-kelompok
kelontang-kelantung
keluar-masuk
kelurahan-kelurahan
kelusuh-kelasah
kelut-melut
kemak-kemik
kemalu-maluan
kemana-mana
kemanja-manjaan
kemarah-marahan
kemasam-masaman
kemati-matian
kembang-kembang
kemenpan-rb
kementerian-kementerian
kemerah-merahan
kempang-kempis
kempas-kempis
kemuda-mudaan
kena-mengena
kenal-mengenal
kenang-kenangan
kencang-kencung
kencing-mengencingi
kencrang-kencring
kendang-kendang
kendang-kendangan
keningrat-ningratan
kentung-kentung
kenyat-kenyit
kepala-kepala
kepala-kepalaan
kepandir-pandiran
kepang-kepot
keperak-perakan
kepetah-lidahan
kepilu-piluan
keping-keping
kepucat-pucatan
kepuh-kepuh
kepura-puraan
keputih-putihan
kerah-kerahan
kerancak-rancakan
kerang-kerangan
kerang-keroh
kerang-kerot
kerang-keruk
kerang-kerung
kerap-kerap
keras-mengerasi
kercap-kercip
kercap-kercup
keriang-keriut
kerja-kerja
kernyat-kernyut
kerobak-kerabit
kerobak-kerobek
kerobak-kerobik
kerobat-kerabit
kerong-kerong
keropas-kerapis
kertak-kertuk
kertap-kertap
keruntang-pungkang
kesalahan-kesalahan
kesap-kesip
kesemena-menaan
kesenak-senakan
kesewenang-wenangan
kesia-siaan
kesik-kesik
kesipu-sipuan
kesu-kesi
kesuh-kesih
kesuk-kesik
ketakar-keteker
ketakutan-ketakutan
ketap-ketap
ketap-ketip
ketar-ketir
ketentuan-ketentuan
ketergesa-gesaan
keti-keti
ketidur-tiduran
ketiga-tiganya
ketir-ketir
ketua-ketua
ketua-tuaan
ketuan-tuanan
keungu-unguan
kewangi-wangian
ki-ka
kia-kia
kiai-kiai
kiak-kiak
kial-kial
kiang-kiut
kiat-kiat
kibang-kibut
kicang-kecoh
kicang-kicu
kick-off
kida-kida
kijang-kijang
kilau-mengilau
kili-kili
kilik-kilik
kincir-kincir
kios-kios
kira-kira
kira-kiraan
kiri-kanan
kirim-berkirim
kisah-kisah
kisi-kisi
kitab-kitab
kitang-kitang
kiu-kiu
klaim-klaim
klik-klikan
klip-klip
klub-klub
kluntang-klantung
knock-knock
knock-on
knock-out
ko-as
ko-pilot
koak-koak
koboi-koboian
kocah-kacih
kocar-kacir
kodam-kodam
kode-kode
kodim-kodim
kodok-kodok
kolang-kaling
kole-kole
koleh-koleh
kolong-kolong
koma-koma
komat-kamit
komisaris-komisaris
komisi-komisi
komite-komite
komoditas-komoditas
kongko-kongko
konsulat-konsulat
konsultan-konsultan
kontal-kantil
kontang-kanting
kontra-terorisme
kontrak-kontrak
konvensi-konvensi
kopat-kapit
koperasi-koperasi
kopi-kopi
koran-koran
koreng-koreng
kos-kosan
kosak-kasik
kota-kota
kota-wakil
kotak-katik
kotak-kotak
koyak-koyak
kuas-kuas
kuat-kuat
kubu-kubuan
kucar-kacir
kucing-kucing
kucing-kucingan
kuda-kuda
kuda-kudaan
kudap-kudap
kue-kue
kulah-kulah
kulak-kulak
kulik-kulik
kulum-kulum
kumat-kamit
kumpul-kumpul
kunang-kunang
kunar-kunar
kung-fu
kuning-hitam
kupat-kapit
kupu-kupu
kura-kura
kurang-kurang
kusat-mesat
kutat-kutet
kuti-kuti
kuwung-kuwung
kyai-kyai
laba-laba
labi-labi
labu-labu
laga-laga
lagi-lagi
lagu-lagu
laguh-lagah
lain-lain
laki-laki
lalu-lalang
lalu-lintas
lama-kelamaan
lama-lama
lamat-lamat
lambat-lambat
lampion-lampion
lampu-lampu
lancang-lancang
lancar-lancar
langak-longok
langgar-melanggar
langit-langit
langkah-langka
langkah-langkah
lanja-lanjaan
lapas-lapas
lapat-lapat
laporan-laporan
laptop-tablet
large-scale
lari-lari
lari-larian
laskar-laskar
lauk-pauk
laun-laun
laut-timur
lawah-lawah
lawak-lawak
lawan-lawan
lawi-lawi
layang-layang
layu-layuan
lebih-lebih
lecet-lecet
legak-legok
legum-legum
legup-legup
leha-leha
lekak-lekuk
lekap-lekup
lekas-lekas
lekat-lekat
lekuh-lekih
lekum-lekum
lekup-lekap
lembaga-lembaga
lempar-lemparan
lenggak-lenggok
lenggok-lenggok
lenggut-lenggut
lengket-lengket
lentam-lentum
lentang-lentok
lentang-lentung
lepa-lepa
lerang-lerang
lereng-lereng
lese-majeste
letah-letai
lete-lete
letuk-letuk
letum-letum
letup-letup
leyeh-leyeh
liang-liuk
liang-liut
liar-liar
liat-liut
lidah-lidah
life-toxins
liga-liga
light-emitting
lika-liku
lil-alamin
lilin-lilin
line-up
lintas-selat
lipat-melipat
liquid-cooled
lithium-ion
lithium-polymer
liuk-liuk
liung-liung
lobi-lobi
lock-up
locked-in
lokasi-lokasi
long-term
longak-longok
lontang-lanting
lontang-lantung
lopak-lapik
lopak-lopak
low-cost
low-density
low-end
low-light
low-multi
low-pass
lucu-lucu
luka-luka
lukisan-lukisan
lumba-lumba
lumi-lumi
luntang-lantung
lupa-lupa
lupa-lupaan
lurah-camat
maaf-memaafkan
mabuk-mabukan
mabul-mabul
macam-macam
macan-macanan
machine-to-machine
mafia-mafia
mahasiswa-mahasiswi
mahasiswa/i
mahi-mahi
main-main
main-mainan
main-mainlah
majelis-majelis
maju-mundur
makam-makam
makan-makan
makan-makanan
makanan-red
make-up
maki-maki
maki-makian
mal-mal
malai-malai
malam-malam
malar-malar
malas-malasan
mali-mali
malu-malu
mama-mama
man-in-the-middle
mana-mana
manajer-manajer
manik-manik
manis-manis
manis-manisan
marah-marah
mark-up
mas-mas
masa-masa
masak-masak
masalah-masalah
mash-up
masing-masing
masjid-masjid
masuk-keluar
mat-matan
mata-mata
match-fixing
mati-mati
mati-matian
maya-maya
mayat-mayat
mayday-mayday
media-media
mega-bintang
mega-tsunami
megal-megol
megap-megap
meger-meger
megrek-megrek
melak-melak
melambai-lambai
melambai-lambaikan
melambat-lambatkan
melaun-laun
melawak-lawak
melayang-layang
melayap-layap
melayap-layapkan
melebih-lebihi
melebih-lebihkan
melejang-lejangkan
melek-melekan
meleleh-leleh
melengah-lengah
melihat-lihat
melimpah-limpah
melincah-lincah
meliuk-liuk
melolong-lolong
melompat-lompat
meloncat-loncat
melonco-lonco
melongak-longok
melonjak-lonjak
memacak-macak
memada-madai
memadan-madan
memaki-maki
memaksa-maksa
memanas-manasi
memancit-mancitkan
memandai-mandai
memanggil-manggil
memanis-manis
memanjut-manjut
memantas-mantas
memasak-masak
memata-matai
mematah-matah
mematuk-matuk
mematut-matut
memau-mau
memayah-mayahkan
membaca-baca
membacah-bacah
membagi-bagikan
membalik-balik
membangkit-bangkit
membarut-barut
membawa-bawa
membayang-bayangi
membayang-bayangkan
membeda-bedakan
membelai-belai
membeli-beli
membelit-belitkan
membelu-belai
membenar-benar
membenar-benari
memberai-beraikan
membesar-besar
membesar-besarkan
membikin-bikin
membilah-bilah
membolak-balikkan
membongkar-bangkir
membongkar-bongkar
membuang-buang
membuat-buat
membulan-bulani
membunga-bungai
membungkuk-bungkuk
memburu-buru
memburu-burukan
memburuk-burukkan
memelintir-melintir
memencak-mencak
memencar-mencar
memercik-mercik
memetak-metak
memetang-metangkan
memetir-metir
memijar-mijar
memikir-mikir
memikir-mikirkan
memilih-milih
memilin-milin
meminang-minang
meminta-minta
memisah-misahkan
memontang-mantingkan
memorak-perandakan
memorak-porandakan
memotong-motong
memperamat-amat
memperamat-amatkan
memperbagai-bagaikan
memperganda-gandakan
memperganduh-ganduhkan
memperimpit-impitkan
memperkuda-kudakan
memperlengah-lengah
memperlengah-lengahkan
mempermacam-macamkan
memperolok-olok
memperolok-olokkan
mempersama-samakan
mempertubi-tubi
mempertubi-tubikan
memperturut-turutkan
memuja-muja
memukang-mukang
memulun-mulun
memundi-mundi
memundi-mundikan
memutar-mutar
memuyu-muyu
men-tweet
menagak-nagak
menakut-nakuti
menang-kalah
menanjur-nanjur
menanti-nanti
menari-nari
mencabik-cabik
mencabik-cabikkan
mencacah-cacah
mencaing-caing
mencak-mencak
mencakup-cakup
mencapak-capak
mencari-cari
mencarik-carik
mencarik-carikkan
mencarut-carut
mencengis-cengis
mencepak-cepak
mencepuk-cepuk
mencerai-beraikan
mencetai-cetai
menciak-ciak
menciap-ciap
menciar-ciar
mencita-citakan
mencium-cium
menciut-ciut
mencla-mencle
mencoang-coang
mencoba-coba
mencocok-cocok
mencolek-colek
menconteng-conteng
mencubit-cubit
mencucuh-cucuh
mencucuh-cucuhkan
mencuri-curi
mendecap-decap
mendegam-degam
mendengar-dengar
mendengking-dengking
mendengus-dengus
mendengut-dengut
menderai-deraikan
menderak-derakkan
menderau-derau
menderu-deru
mendesas-desuskan
mendesus-desus
mendetap-detap
mendewa-dewakan
mendudu-dudu
menduga-duga
menebu-nebu
menegur-neguri
menepak-nepak
menepak-nepakkan
mengabung-ngabung
mengaci-acikan
mengacu-acu
mengada-ada
mengada-ngada
mengadang-adangi
mengaduk-aduk
mengagak-agak
mengagak-agihkan
mengagut-agut
mengais-ngais
mengalang-alangi
mengali-ali
mengalur-alur
mengamang-amang
mengamat-amati
mengambai-ambaikan
mengambang-ambang
mengambung-ambung
mengambung-ambungkan
mengamit-ngamitkan
mengancai-ancaikan
mengancak-ancak
mengancar-ancar
mengangan-angan
mengangan-angankan
mengangguk-angguk
menganggut-anggut
mengangin-anginkan
mengangkat-angkat
menganjung-anjung
menganjung-anjungkan
mengap-mengap
mengapa-apai
mengapi-apikan
mengarah-arahi
mengarang-ngarang
mengata-ngatai
mengatup-ngatupkan
mengaum-aum
mengaum-aumkan
mengejan-ejan
mengejar-ngejar
mengejut-ngejuti
mengelai-ngelai
mengelepik-ngelepik
mengelip-ngelip
mengelu-elukan
mengelus-elus
mengembut-embut
mengempas-empaskan
mengenap-enapkan
mengendap-endap
mengenjak-enjak
mengentak-entak
mengentak-entakkan
mengepak-ngepak
mengepak-ngepakkan
mengepal-ngepalkan
mengerjap-ngerjap
mengerling-ngerling
mengertak-ngertakkan
mengesot-esot
menggaba-gabai
menggali-gali
menggalur-galur
menggamak-gamak
menggamit-gamitkan
menggapai-gapai
menggapai-gapaikan
menggaruk-garuk
menggebu-gebu
menggebyah-uyah
menggeleng-gelengkan
menggelepar-gelepar
menggelepar-geleparkan
menggeliang-geliutkan
menggelinding-gelinding
menggemak-gemak
menggembar-gemborkan
menggerak-gerakkan
menggerecak-gerecak
menggesa-gesakan
menggili-gili
menggodot-godot
menggolak-galikkan
menggorek-gorek
menggoreng-goreng
menggosok-gosok
menggoyang-goyangkan
mengguit-guit
menghalai-balaikan
menghalang-halangi
menghambur-hamburkan
menghinap-hinap
menghitam-memutihkan
menghitung-hitung
menghubung-hubungkan
menghujan-hujankan
mengiang-ngiang
mengibar-ngibarkan
mengibas-ngibas
mengibas-ngibaskan
mengidam-idamkan
mengilah-ngilahkan
mengilai-ilai
mengilat-ngilatkan
mengilik-ngilik
mengimak-imak
mengimbak-imbak
mengiming-iming
mengincrit-incrit
mengingat-ingat
menginjak-injak
mengipas-ngipas
mengira-ngira
mengira-ngirakan
mengiras-iras
mengiras-irasi
mengiris-iris
mengitar-ngitar
mengitik-ngitik
mengodol-odol
mengogok-ogok
mengolak-alik
mengolak-alikkan
mengolang-aling
mengolang-alingkan
mengoleng-oleng
mengolok-olok
mengombang-ambing
mengombang-ambingkan
mengongkang-ongkang
mengongkok-ongkok
mengonyah-anyih
mengopak-apik
mengorak-arik
mengorat-oret
mengorek-ngorek
mengoret-oret
mengorok-orok
mengotak-atik
mengotak-ngatikkan
mengotak-ngotakkan
mengoyak-ngoyak
mengoyak-ngoyakkan
mengoyak-oyak
menguar-nguarkan
menguar-uarkan
mengubah-ubah
mengubek-ubek
menguber-uber
mengubit-ubit
mengubrak-abrik
mengucar-ngacirkan
mengucek-ngucek
mengucek-ucek
menguik-uik
menguis-uis
mengulang-ulang
mengulas-ulas
mengulit-ulit
mengulum-ngulum
mengulur-ulur
menguman-uman
mengumbang-ambingkan
mengumpak-umpak
mengungkat-ungkat
mengungkit-ungkit
mengupa-upa
mengurik-urik
mengusil-usil
mengusil-usilkan
mengutak-atik
mengutak-ngatikkan
mengutik-ngutik
mengutik-utik
menika-nika
menimang-nimang
menimbang-nimbang
menimbun-nimbun
menimpang-nimpangkan
meningkat-ningkat
meniru-niru
menit-menit
menitar-nitarkan
meniup-niup
menjadi-jadi
menjadi-jadikan
menjedot-jedotkan
menjelek-jelekkan
menjengek-jengek
menjengit-jengit
menjerit-jerit
menjilat-jilat
menjungkat-jungkit
menko-menko
menlu-menlu
menonjol-nonjolkan
mentah-mentah
mentang-mentang
menteri-menteri
mentul-mentul
menuding-nuding
menumpah-numpahkan
menunda-nunda
menunduk-nunduk
menusuk-nusuk
menyala-nyala
menyama-nyama
menyama-nyamai
menyambar-nyambar
menyangkut-nyangkutkan
menyanjung-nyanjung
menyanjung-nyanjungkan
menyapu-nyapu
menyarat-nyarat
menyayat-nyayat
menyedang-nyedang
menyedang-nyedangkan
menyelang-nyelangkan
menyelang-nyeling
menyelang-nyelingkan
menyenak-nyenak
menyendi-nyendi
menyentak-nyentak
menyentuh-nyentuh
menyepak-nyepakkan
menyerak-nyerakkan
menyeret-nyeret
menyeru-nyerukan
menyetel-nyetel
menyia-nyiakan
menyibak-nyibak
menyobek-nyobek
menyorong-nyorongkan
menyungguh-nyungguhi
menyuruk-nyuruk
meraba-raba
merah-hitam
merah-merah
merambang-rambang
merangkak-rangkak
merasa-rasai
merata-ratakan
meraung-raung
meraung-raungkan
merayau-rayau
merayu-rayu
mercak-mercik
mercedes-benz
merek-merek
mereka-mereka
mereka-reka
merelap-relap
merem-merem
meremah-remah
meremas-remas
meremeh-temehkan
merempah-rempah
merempah-rempahi
merengek-rengek
merengeng-rengeng
merenik-renik
merenta-renta
merenyai-renyai
meresek-resek
merintang-rintang
merintik-rintik
merobek-robek
meronta-ronta
meruap-ruap
merubu-rubu
merungus-rungus
merungut-rungut
meta-analysis
metode-metode
mewanti-wanti
mewarna-warnikan
meyakin-yakini
mid-range
mid-size
miju-miju
mikro-kecil
mimpi-mimpi
minggu-minggu
minta-minta
minuman-minuman
mixed-use
mobil-mobil
mobile-first
mobile-friendly
moga-moga
mola-mola
momen-momen
mondar-mandir
monyet-monyet
morak-marik
morat-marit
move-on
muda-muda
muda-mudi
muda/i
mudah-mudahan
muka-muka
mula-mula
multiple-output
muluk-muluk
mulut-mulutan
mumi-mumi
mundur-mundur
muntah-muntah
murid-muridnya
musda-musda
museum-museum
muslim-muslimah
musuh-musuh
musuh-musuhnya
nabi-nabi
nada-nadanya
naga-naga
naga-naganya
naik-naik
naik-turun
nakal-nakalan
nama-nama
nanti-nantian
nanya-nanya
nasi-nasi
nasib-nasiban
near-field
negara-negara
negera-negara
negeri-negeri
negeri-red
neka-neka
nekat-nekat
neko-neko
nenek-nenek
neo-liberalisme
next-gen
next-generation
ngeang-ngeang
ngeri-ngeri
nggak-nggak
ngobrol-ngobrol
ngumpul-ngumpul
nilai-nilai
nine-dash
nipa-nipa
nong-nong
norma-norma
novel-novel
nyai-nyai
nyolong-nyolong
nyut-nyutan
ob-gyn
obat-obat
obat-obatan
objek-objek
obok-obok
obrak-abrik
octa-core
odong-odong
oedipus-kompleks
off-road
ogah-agih
ogah-ogah
ogah-ogahan
ogak-agik
ogak-ogak
ogoh-ogoh
olak-alik
olak-olak
olang-aling
olang-alingan
ole-ole
oleh-oleh
olok-olok
olok-olokan
olong-olong
om-om
ombang-ambing
omni-channel
on-board
on-demand
on-fire
on-line
on-off
on-premises
on-roll
on-screen
on-the-go
onde-onde
ondel-ondel
ondos-ondos
one-click
one-to-one
one-touch
one-two
oneng-oneng
ongkang-ongkang
ongol-ongol
online-to-offline
ontran-ontran
onyah-anyih
onyak-anyik
opak-apik
opsi-opsi
opt-in
orak-arik
orang-aring
orang-orang
orang-orangan
orat-oret
organisasi-organisasi
ormas-ormas
orok-orok
orong-orong
oseng-oseng
otak-atik
otak-otak
otak-otakan
over-heating
over-the-air
over-the-top
pa-pa
pabrik-pabrik
padi-padian
pagi-pagi
pagi-sore
pajak-pajak
paket-paket
palas-palas
palato-alveolar
paling-paling
palu-arit
palu-memalu
panas-dingin
panas-panas
pandai-pandai
pandang-memandang
panel-panel
pangeran-pangeran
panggung-panggung
pangkalan-pangkalan
panja-panja
panji-panji
pansus-pansus
pantai-pantai
pao-pao
para-para
parang-parang
parpol-parpol
partai-partai
paru-paru
pas-pasan
pasal-pasal
pasang-memasang
pasang-surut
pasar-pasar
pasu-pasu
paus-paus
paut-memaut
pay-per-click
paya-paya
pdi-p
pecah-pecah
pecat-pecatan
peer-to-peer
pejabat-pejabat
pekak-pekak
pekik-pekuk
pelabuhan-pelabuhan
pelacur-pelacur
pelajar-pelajar
pelan-pelan
pelangi-pelangi
pem-bully
pemain-pemain
pemata-mataan
pemda-pemda
pemeluk-pemeluknya
pemerintah-pemerintah
pemerintah-red
pemerintah-swasta
pemetang-metangan
pemilu-pemilu
pemimpin-pemimpin
peminta-minta
pemuda-pemuda
pemuda-pemudi
penanggung-jawab
pengali-ali
pengaturan-pengaturan
penggembar-gemboran
pengorak-arik
pengotak-ngotakan
pengundang-undang
pengusaha-pengusaha
pentung-pentungan
penyakit-penyakit
perak-perak
perang-perangan
peras-perus
peraturan-peraturan
perda-perda
perempat-final
perempuan-perempuan
pergi-pergi
pergi-pulang
perintang-rintang
perkereta-apian
perlahan-lahan
perlip-perlipan
permen-permen
pernak-pernik
pernik-pernik
pertama-tama
pertandingan-pertandingan
pertimbangan-pertimbangan
perudang-undangan
perundang-undangan
perundangan-undangan
perusahaan-perusahaan
perusahaan-perusahan
perwakilan-perwakilan
pesan-pesan
pesawat-pesawat
peta-jalan
petang-petang
petantang-petenteng
petatang-peteteng
pete-pete
piala-piala
piat-piut
pick-up
picture-in-picture
pihak-pihak
pijak-pijak
pijar-pijar
pijat-pijat
pikir-pikir
pil-pil
pilah-pilih
pilih-pilih
pilihan-pilihan
pilin-memilin
pilkada-pilkada
pina-pina
pindah-pindah
ping-pong
pinjam-meminjam
pintar-pintarlah
pisang-pisang
pistol-pistolan
piting-memiting
planet-planet
play-off
plin-plan
plintat-plintut
plonga-plongo
plug-in
plus-minus
plus-plus
poco-poco
pohon-pohonan
poin-poin
point-of-sale
point-of-sales
pokemon-pokemon
pokja-pokja
pokok-pokok
pokrol-pokrolan
polang-paling
polda-polda
poleng-poleng
polong-polongan
polres-polres
polsek-polsek
polwan-polwan
poma-poma
pondok-pondok
ponpes-ponpes
pontang-panting
pop-up
porak-parik
porak-peranda
porak-poranda
pos-pos
posko-posko
potong-memotong
praktek-praktek
praktik-praktik
produk-produk
program-program
promosi-degradasi
provinsi-provinsi
proyek-proyek
puing-puing
puisi-puisi
puji-pujian
pukang-pukang
pukul-memukul
pulang-pergi
pulau-pulai
pulau-pulau
pull-up
pulut-pulut
pundi-pundi
pungak-pinguk
punggung-memunggung
pura-pura
puruk-parak
pusar-pusar
pusat-pusat
push-to-talk
push-up
push-ups
pusing-pusing
puskesmas-puskesmas
putar-putar
putera-puteri
putih-hitam
putih-putih
putra-putra
putra-putri
putra/i
putri-putri
putus-putus
putusan-putusan
puvi-puvi
quad-core
raba-rabaan
raba-rubu
rada-rada
radio-frequency
ragu-ragu
rahasia-rahasiaan
raja-raja
rama-rama
ramai-ramai
ramalan-ramalan
rambeh-rambeh
rambu-rambu
rame-rame
ramu-ramuan
randa-rondo
rangkul-merangkul
rango-rango
rap-rap
rasa-rasanya
rata-rata
raun-raun
read-only
real-life
real-time
rebah-rebah
rebah-rebahan
rebas-rebas
red-eye
redam-redam
redep-redup
rehab-rekon
reja-reja
reka-reka
reka-rekaan
rekan-rekan
rekan-rekannya
rekor-rekor
relief-relief
remah-remah
remang-remang
rembah-rembah
rembah-rembih
remeh-cemeh
remeh-temeh
rempah-rempah
rencana-rencana
renyai-renyai
rep-repan
repot-repot
repuh-repuh
restoran-restoran
retak-retak
riang-riang
ribu-ribu
ribut-ribut
rica-rica
ride-sharing
rigi-rigi
rinai-rinai
rintik-rintik
ritual-ritual
robak-rabik
robat-rabit
robot-robot
role-play
role-playing
roll-on
rombang-rambing
romol-romol
rompang-romping
rondah-rondih
ropak-rapik
royal-royalan
royo-royo
ruak-ruak
ruba-ruba
rudal-rudal
ruji-ruji
ruku-ruku
rumah-rumah
rumah-rumahan
rumbai-rumbai
rumput-rumputan
runding-merunding
rundu-rundu
runggu-rangga
runner-up
runtang-runtung
rupa-rupa
rupa-rupanya
rusun-rusun
rute-rute
saat-saat
saban-saban
sabu-sabu
sabung-menyabung
sah-sah
sahabat-sahabat
saham-saham
sahut-menyahut
saing-menyaing
saji-sajian
sakit-sakitan
saksi-saksi
saku-saku
salah-salah
sama-sama
samar-samar
sambar-menyambar
sambung-bersambung
sambung-menyambung
sambut-menyambut
samo-samo
sampah-sampah
sampai-sampai
samping-menyamping
sana-sini
sandar-menyandar
sandi-sandi
sangat-sangat
sangkut-menyangkut
sapa-menyapa
sapai-sapai
sapi-sapi
sapu-sapu
saran-saran
sarana-prasarana
sari-sari
sarit-sarit
satu-dua
satu-satu
satu-satunya
satuan-satuan
saudara-saudara
sauk-menyauk
sauk-sauk
sayang-sayang
sayap-sayap
sayup-menyayup
sayup-sayup
sayur-mayur
sayur-sayuran
sci-fi
seagak-agak
seakal-akal
seakan-akan
sealak-alak
seari-arian
sebaik-baiknya
sebelah-menyebelah
sebentar-sebentar
seberang-menyeberang
seberuntung-beruntungnya
sebesar-besarnya
seboleh-bolehnya
sedalam-dalamnya
sedam-sedam
sedang-menyedang
sedang-sedang
sedap-sedapan
sedapat-dapatnya
sedikit-dikitnya
sedikit-sedikit
sedikit-sedikitnya
sedini-dininya
seelok-eloknya
segala-galanya
segan-menyegan
segan-menyegani
segan-segan
sehabis-habisnya
sehari-hari
sehari-harian
sehari-harinya
sejadi-jadinya
sekali-kali
sekali-sekali
sekenyang-kenyangnya
sekira-kira
sekolah-sekolah
sekonyong-konyong
sekosong-kosongnya
sektor-sektor
sekuasa-kuasanya
sekuat-kuatnya
sekurang-kurangnya
sel-sel
sela-menyela
sela-sela
selak-seluk
selama-lamanya
selambat-lambatnya
selang-seli
selang-seling
selar-belar
selat-latnya
selatan-tenggara
selekas-lekasnya
selentang-selenting
selepas-lepas
self-driving
self-esteem
self-healing
self-help
selir-menyelir
seloyong-seloyong
seluk-beluk
seluk-semeluk
sema-sema
semah-semah
semak-semak
semaksimal-maksimalnya
semalam-malaman
semang-semang
semanis-manisnya
semasa-masa
semata-mata
semau-maunya
sembunyi-sembunyi
sembunyi-sembunyian
sembur-sembur
semena-mena
semenda-menyemenda
semengga-mengga
semenggah-menggah
sementang-mentang
semerdeka-merdekanya
semi-final
semi-permanen
sempat-sempatnya
semu-semu
semua-muanya
semujur-mujurnya
semut-semutan
sen-senan
sendiri-sendiri
sengal-sengal
sengar-sengir
sengau-sengauan
senggak-sengguk
senggang-tenggang
senggol-menyenggol
senior-junior
senjata-senjata
senyum-senyum
seolah-olah
sepala-pala
sepandai-pandai
sepetang-petangan
sepoi-sepoi
sepraktis-praktisnya
sepuas-puasnya
serak-serak
serak-serik
serang-menyerang
serang-serangan
serangan-serangan
seraya-menyeraya
serba-serbi
serbah-serbih
serembah-serembih
serigala-serigala
sering-sering
serobot-serobotan
serong-menyerong
serta-menyertai
serta-merta
serta-serta
seru-seruan
service-oriented
sesak-menyesak
sesal-menyesali
sesayup-sayup
sesi-sesi
sesuang-suang
sesudah-sudah
sesudah-sudahnya
sesuka-suka
sesuka-sukanya
set-piece
setempat-setempat
setengah-setengah
setidak-tidaknya
setinggi-tingginya
seupaya-upaya
seupaya-upayanya
sewa-menyewa
sewaktu-waktu
sewenang-wenang
sewot-sewotan
shabu-shabu
short-term
short-throw
sia-sia
siang-siang
siap-siap
siapa-siapa
sibar-sibar
sibur-sibur
sida-sida
side-by-side
sign-in
siku-siku
sikut-sikutan
silah-silah
silang-menyilang
silir-semilir
simbol-simbol
simpan-pinjam
sinar-menyinar
sinar-seminar
sinar-suminar
sindir-menyindir
singa-singa
singgah-menyinggah
single-core
sipil-militer
sir-siran
sirat-sirat
sisa-sisa
sisi-sisi
siswa-siswa
siswa-siswi
siswa/i
siswi-siswi
situ-situ
situs-situs
six-core
six-speed
slintat-slintut
slo-mo
slow-motion
snap-on
sobek-sobekan
sodok-sodokan
sok-sokan
solek-menyolek
solid-state
sorak-sorai
sorak-sorak
sore-sore
sosio-ekonomi
soya-soya
spill-resistant
split-screen
sponsor-sponsor
sponsor-sponsoran
srikandi-srikandi
staf-staf
stand-by
stand-up
start-up
stasiun-stasiun
state-owned
striker-striker
studi-studi
suam-suam
suami-isteri
suami-istri
suami-suami
suang-suang
suara-suara
sudin-sudin
sudu-sudu
sudung-sudung
sugi-sugi
suka-suka
suku-suku
sulang-menyulang
sulat-sulit
sulur-suluran
sum-sum
sumber-sumber
sumpah-sumpah
sumpit-sumpit
sundut-bersundut
sungai-sungai
sungguh-sungguh
sungut-sungut
sunting-menyunting
super-damai
super-rahasia
super-sub
supply-demand
supply-side
suram-suram
surat-menyurat
surat-surat
suruh-suruhan
suruk-surukan
susul-menyusul
suwir-suwir
syarat-syarat
system-on-chip
t-shirt
t-shirts
tabar-tabar
tabir-mabir
tabrak-tubruk
tabuh-tabuhan
tabun-menabun
tahu-menahu
tahu-tahu
tahun-tahun
takah-takahnya
takang-takik
take-off
takut-takut
takut-takutan
tali-bertali
tali-tali
talun-temalun
taman-taman
tampak-tampak
tanak-tanakan
tanam-menanam
tanam-tanaman
tanda-tanda
tangan-menangan
tangan-tangan
tangga-tangga
tanggal-tanggal
tanggul-tanggul
tanggung-menanggung
tanggung-tanggung
tank-tank
tante-tante
tanya-jawab
tapa-tapa
tapak-tapak
tari-menari
tari-tarian
tarik-menarik
tarik-ulur
tata-tertib
tatah-tatah
tau-tau
tawa-tawa
tawak-tawak
tawang-tawang
tawar-menawar
tawar-tawar
tayum-temayum
tebak-tebakan
tebu-tebu
tedong-tedong
tegak-tegak
tegerbang-gerbang
teh-tehan
tek-tek
teka-teki
teknik-teknik
teman-teman
teman-temanku
temas-temas
tembak-menembak
temeh-temeh
tempa-menempa
tempat-tempat
tempo-tempo
temut-temut
tenang-tenang
tengah-tengah
tenggang-menenggang
tengok-menengok
teori-teori
teraba-raba
teralang-alang
terambang-ambang
terambung-ambung
terang-terang
terang-terangan
teranggar-anggar
terangguk-angguk
teranggul-anggul
terangin-angin
terangkup-angkup
teranja-anja
terapung-apung
terayan-rayan
terayap-rayap
terbada-bada
terbahak-bahak
terbang-terbang
terbata-bata
terbatuk-batuk
terbayang-bayang
terbeda-bedakan
terbengkil-bengkil
terbengong-bengong
terbirit-birit
terbuai-buai
terbuang-buang
terbungkuk-bungkuk
terburu-buru
tercangak-cangak
tercengang-cengang
tercilap-cilap
tercongget-congget
tercoreng-moreng
tercungap-cungap
terdangka-dangka
terdengih-dengih
terduga-duga
terekeh-ekeh
terembut-embut
terembut-rembut
terempas-empas
terengah-engah
teresak-esak
tergagap-gagap
tergagau-gagau
tergaguk-gaguk
tergapai-gapai
tergegap-gegap
tergegas-gegas
tergelak-gelak
tergelang-gelang
tergeleng-geleng
tergelung-gelung
tergerai-gerai
tergerenyeng-gerenyeng
tergesa-gesa
tergila-gila
tergolek-golek
tergontai-gontai
tergudik-gudik
tergugu-gugu
terguling-guling
tergulut-gulut
terhambat-hambat
terharak-harak
terharap-harap
terhengit-hengit
terheran-heran
terhinggut-hinggut
terigau-igau
terimpi-impi
terincut-incut
teringa-inga
teringat-ingat
terinjak-injak
terisak-isak
terjembak-jembak
terjerit-jerit
terkadang-kadang
terkagum-kagum
terkaing-kaing
terkakah-kakah
terkakak-kakak
terkampul-kampul
terkanjar-kanjar
terkantuk-kantuk
terkapah-kapah
terkapai-kapai
terkapung-kapung
terkatah-katah
terkatung-katung
terkecap-kecap
terkedek-kedek
terkedip-kedip
terkejar-kejar
terkekau-kekau
terkekeh-kekeh
terkekek-kekek
terkelinjat-kelinjat
terkelip-kelip
terkempul-kempul
terkemut-kemut
terkencar-kencar
terkencing-kencing
terkentut-kentut
terkepak-kepak
terkesot-kesot
terkesut-kesut
terkial-kial
terkijai-kijai
terkikih-kikih
terkikik-kikik
terkincak-kincak
terkindap-kindap
terkinja-kinja
terkirai-kirai
terkitar-kitar
terkocoh-kocoh
terkojol-kojol
terkokol-kokol
terkosel-kosel
terkotak-kotak
terkoteng-koteng
terkuai-kuai
terkumpal-kumpal
terlara-lara
terlayang-layang
terlebih-lebih
terlincah-lincah
terliuk-liuk
terlolong-lolong
terlongong-longong
terlunta-lunta
termangu-mangu
termanja-manja
termata-mata
termengah-mengah
termenung-menung
termimpi-mimpi
termonyong-monyong
ternanti-nanti
terngiang-ngiang
teroleng-oleng
terombang-ambing
terpalit-palit
terpandang-pandang
terpecah-pecah
terpekik-pekik
terpencar-pencar
terpereh-pereh
terpijak-pijak
terpikau-pikau
terpilah-pilah
terpinga-pinga
terpingkal-pingkal
terpingkau-pingkau
terpontang-panting
terpusing-pusing
terputus-putus
tersanga-sanga
tersaruk-saruk
tersedan-sedan
tersedih-sedih
tersedu-sedu
terseduh-seduh
tersendat-sendat
tersendeng-sendeng
tersengal-sengal
tersengguk-sengguk
tersengut-sengut
terseok-seok
tersera-sera
terserak-serak
tersetai-setai
tersia-sia
tersipu-sipu
tersoja-soja
tersungkuk-sungkuk
tersuruk-suruk
tertagak-tagak
tertahan-tahan
tertatih-tatih
tertegun-tegun
tertekan-tekan
terteleng-teleng
tertendang-tendang
tertimpang-timpang
tertitar-titar
terumbang-ambing
terumbang-umbang
terungkap-ungkap
terus-menerus
terus-terusan
tete-a-tete
text-to-speech
think-tank
think-thank
third-party
third-person
three-axis
three-point
tiap-tiap
tiba-tiba
tidak-tidak
tidur-tidur
tidur-tiduran
tie-dye
tie-in
tiga-tiganya
tikam-menikam
tiki-taka
tikus-tikus
tilik-menilik
tim-tim
timah-timah
timang-timangan
timbang-menimbang
time-lapse
timpa-menimpa
timu-timu
timun-timunan
timur-barat
timur-laut
timur-tenggara
tindih-bertindih
tindih-menindih
tinjau-meninjau
tinju-meninju
tip-off
tipu-tipu
tiru-tiruan
titik-titik
titik-titiknya
tiup-tiup
to-do
tokak-takik
toko-toko
tokoh-tokoh
tokok-menokok
tolak-menolak
tolong-menolong
tong-tong
top-level
top-up
totol-totol
touch-screen
trade-in
training-camp
trans-nasional
treble-winner
tri-band
trik-trik
triple-core
truk-truk
tua-tua
tuan-tuan
tuang-tuang
tuban-tuban
tubuh-tubuh
tujuan-tujuan
tuk-tuk
tukang-menukang
tukar-menukar
tulang-belulang
tulang-tulangan
tuli-tuli
tulis-menulis
tumbuh-tumbuhan
tumpang-tindih
tune-up
tunggang-tunggik
tunggang-tungging
tunggang-tunggit
tunggul-tunggul
tunjuk-menunjuk
tupai-tupai
tupai-tupaian
turi-turian
turn-based
turnamen-turnamen
turun-temurun
turut-menurut
turut-turutan
tuyuk-tuyuk
twin-cam
twin-turbocharged
two-state
two-step
two-tone
u-shape
uang-uangan
uar-uar
ubek-ubekan
ubel-ubel
ubrak-abrik
ubun-ubun
ubur-ubur
uci-uci
udang-undang
udap-udapan
ugal-ugalan
uget-uget
uir-uir
ujar-ujar
uji-coba
ujung-ujung
ujung-ujungnya
uka-uka
ukir-mengukir
ukir-ukiran
ula-ula
ulak-ulak
ulam-ulam
ulang-alik
ulang-aling
ulang-ulang
ulap-ulap
ular-ular
ular-ularan
ulek-ulek
ulu-ulu
ulung-ulung
umang-umang
umbang-ambing
umbi-umbian
umbul-umbul
umbut-umbut
uncang-uncit
undak-undakan
undang-undang
undang-undangnya
unduk-unduk
undung-undung
undur-undur
unek-unek
ungah-angih
unggang-anggit
unggat-unggit
unggul-mengungguli
ungkit-ungkit
unit-unit
universitas-universitas
unsur-unsur
untang-anting
unting-unting
untung-untung
untung-untungan
upah-mengupah
upih-upih
upside-down
ura-ura
uran-uran
urat-urat
uring-uringan
urup-urup
urup-urupan
urus-urus
usaha-usaha
user-user
user-useran
utak-atik
utang-piutang
utang-utang
utar-utar
utara-jauh
utara-selatan
uter-uter
utusan-utusan
v-belt
v-neck
value-added
very-very
video-video
visi-misi
visi-misinya
voa-islam
voice-over
volt-ampere
wajah-wajah
wajar-wajar
wake-up
wakil-wakil
walk-in
walk-out
wangi-wangian
wanita-wanita
wanti-wanti
wara-wara
wara-wiri
warna-warna
warna-warni
was-was
water-cooled
web-based
wide-angle
wilayah-wilayah
win-win
wira-wiri
wora-wari
work-life
world-class
yang-yang
yayasan-yayasan
year-on-year
yel-yel
yo-yo
zam-zam
zig-zag
""".split()
)



from .stop_words import STOP_WORDS
from ...language import Language, BaseDefaults


class GujaratiDefaults(BaseDefaults):
    stop_words = STOP_WORDS


class Gujarati(Language):
    lang = "gu"
    Defaults = GujaratiDefaults


__all__ = ["Gujarati"]



import pytest


def test_noun_chunks_is_parsed_de(de_tokenizer):
    """Test that noun_chunks raises Value Error for 'de' language if Doc is not parsed."""
    doc = de_tokenizer("Er lag auf seinem")
    with pytest.raises(ValueError):
        list(doc.noun_chunks)



from thinc.api import Model, normal_init

from ..util import registry


@registry.layers("spacy.PrecomputableAffine.v1")
def PrecomputableAffine(nO, nI, nF, nP, dropout=0.1):
    model = Model(
        "precomputable_affine",
        forward,
        init=init,
        dims={"nO": nO, "nI": nI, "nF": nF, "nP": nP},
        params={"W": None, "b": None, "pad": None},
        attrs={"dropout_rate": dropout},
    )
    return model


def forward(model, X, is_train):
    nF = model.get_dim("nF")
    nO = model.get_dim("nO")
    nP = model.get_dim("nP")
    nI = model.get_dim("nI")
    W = model.get_param("W")
    # Preallocate array for layer output, including padding.
    Yf = model.ops.alloc2f(X.shape[0] + 1, nF * nO * nP, zeros=False)
    model.ops.gemm(X, W.reshape((nF * nO * nP, nI)), trans2=True, out=Yf[1:])
    Yf = Yf.reshape((Yf.shape[0], nF, nO, nP))

    # Set padding. Padding has shape (1, nF, nO, nP). Unfortunately, we cannot
    # change its shape to (nF, nO, nP) without breaking existing models. So
    # we'll squeeze the first dimension here.
    Yf[0] = model.ops.xp.squeeze(model.get_param("pad"), 0)

    def backward(dY_ids):
        # This backprop is particularly tricky, because we get back a different
        # thing from what we put out. We put out an array of shape:
        # (nB, nF, nO, nP), and get back:
        # (nB, nO, nP) and ids (nB, nF)
        # The ids tell us the values of nF, so we would have:
        #
        # dYf = zeros((nB, nF, nO, nP))
        # for b in range(nB):
        #     for f in range(nF):
        #         dYf[b, ids[b, f]] += dY[b]
        #
        # However, we avoid building that array for efficiency -- and just pass
        # in the indices.
        dY, ids = dY_ids
        assert dY.ndim == 3
        assert dY.shape[1] == nO, dY.shape
        assert dY.shape[2] == nP, dY.shape
        # nB = dY.shape[0]
        model.inc_grad("pad", _backprop_precomputable_affine_padding(model, dY, ids))
        Xf = X[ids]
        Xf = Xf.reshape((Xf.shape[0], nF * nI))

        model.inc_grad("b", dY.sum(axis=0))
        dY = dY.reshape((dY.shape[0], nO * nP))

        Wopfi = W.transpose((1, 2, 0, 3))
        Wopfi = Wopfi.reshape((nO * nP, nF * nI))
        dXf = model.ops.gemm(dY.reshape((dY.shape[0], nO * nP)), Wopfi)

        dWopfi = model.ops.gemm(dY, Xf, trans1=True)
        dWopfi = dWopfi.reshape((nO, nP, nF, nI))
        # (o, p, f, i) --> (f, o, p, i)
        dWopfi = dWopfi.transpose((2, 0, 1, 3))
        model.inc_grad("W", dWopfi)
        return dXf.reshape((dXf.shape[0], nF, nI))

    return Yf, backward


def _backprop_precomputable_affine_padding(model, dY, ids):
    nB = dY.shape[0]
    nF = model.get_dim("nF")
    nP = model.get_dim("nP")
    nO = model.get_dim("nO")
    # Backprop the "padding", used as a filler for missing values.
    # Values that are missing are set to -1, and each state vector could
    # have multiple missing values. The padding has different values for
    # different missing features. The gradient of the padding vector is:
    #
    # for b in range(nB):
    #     for f in range(nF):
    #         if ids[b, f] < 0:
    #             d_pad[f] += dY[b]
    #
    # Which can be rewritten as:
    #
    # (ids < 0).T @ dY
    mask = model.ops.asarray(ids < 0, dtype="f")
    d_pad = model.ops.gemm(mask, dY.reshape(nB, nO * nP), trans1=True)
    return d_pad.reshape((1, nF, nO, nP))


def init(model, X=None, Y=None):
    """This is like the 'layer sequential unit variance', but instead
    of taking the actual inputs, we randomly generate whitened data.

    Why's this all so complicated? We have a huge number of inputs,
    and the maxout unit makes guessing the dynamics tricky. Instead
    we set the maxout weights to values that empirically result in
    whitened outputs given whitened inputs.
    """
    if model.has_param("W") and model.get_param("W").any():
        return

    nF = model.get_dim("nF")
    nO = model.get_dim("nO")
    nP = model.get_dim("nP")
    nI = model.get_dim("nI")
    W = model.ops.alloc4f(nF, nO, nP, nI)
    b = model.ops.alloc2f(nO, nP)
    pad = model.ops.alloc4f(1, nF, nO, nP)

    ops = model.ops
    W = normal_init(ops, W.shape, mean=float(ops.xp.sqrt(1.0 / nF * nI)))
    pad = normal_init(ops, pad.shape, mean=1.0)
    model.set_param("W", W)
    model.set_param("b", b)
    model.set_param("pad", pad)

    ids = ops.alloc((5000, nF), dtype="f")
    ids += ops.xp.random.uniform(0, 1000, ids.shape)
    ids = ops.asarray(ids, dtype="i")
    tokvecs = ops.alloc((5000, nI), dtype="f")
    tokvecs += ops.xp.random.normal(loc=0.0, scale=1.0, size=tokvecs.size).reshape(
        tokvecs.shape
    )

    def predict(ids, tokvecs):
        # nS ids. nW tokvecs. Exclude the padding array.
        hiddens = model.predict(tokvecs[:-1])  # (nW, f, o, p)
        vectors = model.ops.alloc((ids.shape[0], nO * nP), dtype="f")
        # need nS vectors
        hiddens = hiddens.reshape((hiddens.shape[0] * nF, nO * nP))
        model.ops.scatter_add(vectors, ids.flatten(), hiddens)
        vectors = vectors.reshape((vectors.shape[0], nO, nP))
        vectors += b
        vectors = model.ops.asarray(vectors)
        if nP >= 2:
            return model.ops.maxout(vectors)[0]
        else:
            return vectors * (vectors >= 0)

    tol_var = 0.01
    tol_mean = 0.01
    t_max = 10
    W = model.get_param("W").copy()
    b = model.get_param("b").copy()
    for t_i in range(t_max):
        acts1 = predict(ids, tokvecs)
        var = model.ops.xp.var(acts1)
        mean = model.ops.xp.mean(acts1)
        if abs(var - 1.0) >= tol_var:
            W /= model.ops.xp.sqrt(var)
            model.set_param("W", W)
        elif abs(mean) >= tol_mean:
            b -= mean
            model.set_param("b", b)
        else:
            break



from typing import Tuple, Callable
from thinc.api import Model, to_numpy
from thinc.types import Ragged, Ints1d

from ..util import registry


@registry.layers("spacy.extract_spans.v1")
def extract_spans() -> Model[Tuple[Ragged, Ragged], Ragged]:
    """Extract spans from a sequence of source arrays, as specified by an array
    of (start, end) indices. The output is a ragged array of the
    extracted spans.
    """
    return Model(
        "extract_spans", forward, layers=[], refs={}, attrs={}, dims={}, init=init
    )


def init(model, X=None, Y=None):
    pass


def forward(
    model: Model, source_spans: Tuple[Ragged, Ragged], is_train: bool
) -> Tuple[Ragged, Callable]:
    """Get subsequences from source vectors."""
    ops = model.ops
    X, spans = source_spans
    assert spans.dataXd.ndim == 2
    indices = _get_span_indices(ops, spans, X.lengths)
    if len(indices) > 0:
        Y = Ragged(X.dataXd[indices], spans.dataXd[:, 1] - spans.dataXd[:, 0])  # type: ignore[arg-type, index]
    else:
        Y = Ragged(
            ops.xp.zeros(X.dataXd.shape, dtype=X.dataXd.dtype),
            ops.xp.zeros((len(X.lengths),), dtype="i"),
        )
    x_shape = X.dataXd.shape
    x_lengths = X.lengths

    def backprop_windows(dY: Ragged) -> Tuple[Ragged, Ragged]:
        dX = Ragged(ops.alloc2f(*x_shape), x_lengths)
        ops.scatter_add(dX.dataXd, indices, dY.dataXd)  # type: ignore[arg-type]
        return (dX, spans)

    return Y, backprop_windows


def _get_span_indices(ops, spans: Ragged, lengths: Ints1d) -> Ints1d:
    """Construct a flat array that has the indices we want to extract from the
    source data. For instance, if we want the spans (5, 9), (8, 10) the
    indices will be [5, 6, 7, 8, 8, 9].
    """
    spans, lengths = _ensure_cpu(spans, lengths)
    indices = []
    offset = 0
    for i, length in enumerate(lengths):
        spans_i = spans[i].dataXd + offset
        for j in range(spans_i.shape[0]):
            indices.append(ops.xp.arange(spans_i[j, 0], spans_i[j, 1]))  # type: ignore[call-overload, index]
        offset += length
    return ops.flatten(indices, dtype="i", ndim_if_empty=1)


def _ensure_cpu(spans: Ragged, lengths: Ints1d) -> Tuple[Ragged, Ints1d]:
    return Ragged(to_numpy(spans.dataXd), to_numpy(spans.lengths)), to_numpy(lengths)



from .stop_words import STOP_WORDS
from .lex_attrs import LEX_ATTRS
from .punctuation import TOKENIZER_INFIXES
from ...language import Language, BaseDefaults


class LugandaDefaults(BaseDefaults):
    lex_attr_getters = LEX_ATTRS
    infixes = TOKENIZER_INFIXES
    stop_words = STOP_WORDS


class Luganda(Language):
    lang = "lg"
    Defaults = LugandaDefaults


__all__ = ["Luganda"]



from typing import Union, Iterable, Dict, Any
from pathlib import Path
import sys

# set library-specific custom warning handling before doing anything else
from .errors import setup_default_warnings

setup_default_warnings()  # noqa: E402

# These are imported as part of the API
from thinc.api import prefer_gpu, require_gpu, require_cpu  # noqa: F401
from thinc.api import Config

from . import pipeline  # noqa: F401
from .cli.info import info  # noqa: F401
from .glossary import explain  # noqa: F401
from .about import __version__  # noqa: F401
from .util import registry, logger  # noqa: F401

from .errors import Errors
from .language import Language
from .vocab import Vocab
from . import util


if sys.maxunicode == 65535:
    raise SystemError(Errors.E130)


def load(
    name: Union[str, Path],
    *,
    vocab: Union[Vocab, bool] = True,
    disable: Union[str, Iterable[str]] = util._DEFAULT_EMPTY_PIPES,
    enable: Union[str, Iterable[str]] = util._DEFAULT_EMPTY_PIPES,
    exclude: Union[str, Iterable[str]] = util._DEFAULT_EMPTY_PIPES,
    config: Union[Dict[str, Any], Config] = util.SimpleFrozenDict(),
) -> Language:
    """Load a spaCy model from an installed package or a local path.

    name (str): Package name or model path.
    vocab (Vocab): A Vocab object. If True, a vocab is created.
    disable (Union[str, Iterable[str]]): Name(s) of pipeline component(s) to disable. Disabled
        pipes will be loaded but they won't be run unless you explicitly
        enable them by calling nlp.enable_pipe.
    enable (Union[str, Iterable[str]]): Name(s) of pipeline component(s) to enable. All other
        pipes will be disabled (but can be enabled later using nlp.enable_pipe).
    exclude (Union[str, Iterable[str]]): Name(s) of pipeline component(s) to exclude. Excluded
        components won't be loaded.
    config (Dict[str, Any] / Config): Config overrides as nested dict or dict
        keyed by section values in dot notation.
    RETURNS (Language): The loaded nlp object.
    """
    return util.load_model(
        name,
        vocab=vocab,
        disable=disable,
        enable=enable,
        exclude=exclude,
        config=config,
    )


def blank(
    name: str,
    *,
    vocab: Union[Vocab, bool] = True,
    config: Union[Dict[str, Any], Config] = util.SimpleFrozenDict(),
    meta: Dict[str, Any] = util.SimpleFrozenDict(),
) -> Language:
    """Create a blank nlp object for a given language code.

    name (str): The language code, e.g. "en".
    vocab (Vocab): A Vocab object. If True, a vocab is created.
    config (Dict[str, Any] / Config): Optional config overrides.
    meta (Dict[str, Any]): Overrides for nlp.meta.
    RETURNS (Language): The nlp object.
    """
    LangClass = util.get_lang_class(name)
    # We should accept both dot notation and nested dict here for consistency
    config = util.dot_to_dict(config)
    return LangClass.from_config(config, vocab=vocab, meta=meta)



from typing import Optional
from pathlib import Path
from wasabi import msg
import typer
import re

from ._util import app, Arg, Opt, parse_config_overrides, show_validation_error
from ._util import import_code, setup_gpu
from ..training.pretrain import pretrain
from ..util import load_config


@app.command(
    "pretrain",
    context_settings={"allow_extra_args": True, "ignore_unknown_options": True},
)
def pretrain_cli(
    # fmt: off
    ctx: typer.Context,  # This is only used to read additional arguments
    config_path: Path = Arg(..., help="Path to config file", exists=True, dir_okay=False, allow_dash=True),
    output_dir: Path = Arg(..., help="Directory to write weights to on each epoch"),
    code_path: Optional[Path] = Opt(None, "--code", "-c", help="Path to Python file with additional code (registered functions) to be imported"),
    resume_path: Optional[Path] = Opt(None, "--resume-path", "-r", help="Path to pretrained weights from which to resume pretraining"),
    epoch_resume: Optional[int] = Opt(None, "--epoch-resume", "-er", help="The epoch to resume counting from when using --resume-path. Prevents unintended overwriting of existing weight files."),
    use_gpu: int = Opt(-1, "--gpu-id", "-g", help="GPU ID or -1 for CPU"),
    # fmt: on
):
    """
    Pre-train the 'token-to-vector' (tok2vec) layer of pipeline components,
    using an approximate language-modelling objective. Two objective types
    are available, vector-based and character-based.

    In the vector-based objective, we load word vectors that have been trained
    using a word2vec-style distributional similarity algorithm, and train a
    component like a CNN, BiLSTM, etc to predict vectors which match the
    pretrained ones. The weights are saved to a directory after each epoch. You
    can then pass a path to one of these pretrained weights files to the
    'spacy train' command.

    This technique may be especially helpful if you have little labelled data.
    However, it's still quite experimental, so your mileage may vary.

    To load the weights back in during 'spacy train', you need to ensure
    all settings are the same between pretraining and training. Ideally,
    this is done by using the same config file for both commands.

    DOCS: https://spacy.io/api/cli#pretrain
    """
    config_overrides = parse_config_overrides(ctx.args)
    import_code(code_path)
    verify_cli_args(config_path, output_dir, resume_path, epoch_resume)
    setup_gpu(use_gpu)
    msg.info(f"Loading config from: {config_path}")

    with show_validation_error(config_path):
        raw_config = load_config(
            config_path, overrides=config_overrides, interpolate=False
        )
    config = raw_config.interpolate()
    if not config.get("pretraining"):
        # TODO: What's the solution here? How do we handle optional blocks?
        msg.fail("The [pretraining] block in your config is empty", exits=1)
    if not output_dir.exists():
        output_dir.mkdir(parents=True)
        msg.good(f"Created output directory: {output_dir}")
    # Save non-interpolated config
    raw_config.to_disk(output_dir / "config.cfg")
    msg.good("Saved config file in the output directory")

    pretrain(
        config,
        output_dir,
        resume_path=resume_path,
        epoch_resume=epoch_resume,
        use_gpu=use_gpu,
        silent=False,
    )
    msg.good("Successfully finished pretrain")


def verify_cli_args(config_path, output_dir, resume_path, epoch_resume):
    if not config_path or (str(config_path) != "-" and not config_path.exists()):
        msg.fail("Config file not found", config_path, exits=1)
    if output_dir.exists() and [p for p in output_dir.iterdir()]:
        if resume_path:
            msg.warn(
                "Output directory is not empty.",
                "If you're resuming a run in this directory, the old weights "
                "for the consecutive epochs will be overwritten with the new ones.",
            )
        else:
            msg.warn(
                "Output directory is not empty. ",
                "It is better to use an empty directory or refer to a new output path, "
                "then the new directory will be created for you.",
            )
    if resume_path is not None:
        if resume_path.is_dir():
            # This is necessary because Windows gives a Permission Denied when we
            # try to open the directory later, which is confusing. See #7878
            msg.fail(
                "--resume-path should be a weights file, but {resume_path} is a directory.",
                exits=True,
            )
        model_name = re.search(r"model\d+\.bin", str(resume_path))
        if not model_name and not epoch_resume:
            msg.fail(
                "You have to use the --epoch-resume setting when using a renamed weight file for --resume-path",
                exits=True,
            )
        elif not model_name and epoch_resume < 0:
            msg.fail(
                f"The argument --epoch-resume has to be greater or equal to 0. {epoch_resume} is invalid",
                exits=True,
            )



from ...symbols import ORTH, NORM


_exc = {}


_abbr_exc = [
    {ORTH: "м", NORM: "метар"},
    {ORTH: "мм", NORM: "милиметар"},
    {ORTH: "цм", NORM: "центиметар"},
    {ORTH: "см", NORM: "сантиметар"},
    {ORTH: "дм", NORM: "дециметар"},
    {ORTH: "км", NORM: "километар"},
    {ORTH: "кг", NORM: "килограм"},
    {ORTH: "дкг", NORM: "декаграм"},
    {ORTH: "дг", NORM: "дециграм"},
    {ORTH: "мг", NORM: "милиграм"},
    {ORTH: "г", NORM: "грам"},
    {ORTH: "т", NORM: "тон"},
    {ORTH: "кл", NORM: "килолитар"},
    {ORTH: "хл", NORM: "хектолитар"},
    {ORTH: "дкл", NORM: "декалитар"},
    {ORTH: "л", NORM: "литар"},
    {ORTH: "дл", NORM: "децилитар"},
]
for abbr in _abbr_exc:
    _exc[abbr[ORTH]] = [abbr]

_abbr_line_exc = [
    {ORTH: "д-р", NORM: "доктор"},
    {ORTH: "м-р", NORM: "магистер"},
    {ORTH: "г-ѓа", NORM: "госпоѓа"},
    {ORTH: "г-ца", NORM: "госпоѓица"},
    {ORTH: "г-дин", NORM: "господин"},
]

for abbr in _abbr_line_exc:
    _exc[abbr[ORTH]] = [abbr]

_abbr_dot_exc = [
    {ORTH: "в.", NORM: "век"},
    {ORTH: "в.д.", NORM: "вршител на должност"},
    {ORTH: "г.", NORM: "година"},
    {ORTH: "г.г.", NORM: "господин господин"},
    {ORTH: "м.р.", NORM: "машки род"},
    {ORTH: "год.", NORM: "женски род"},
    {ORTH: "с.р.", NORM: "среден род"},
    {ORTH: "н.е.", NORM: "наша ера"},
    {ORTH: "о.г.", NORM: "оваа година"},
    {ORTH: "о.м.", NORM: "овој месец"},
    {ORTH: "с.", NORM: "село"},
    {ORTH: "т.", NORM: "точка"},
    {ORTH: "т.е.", NORM: "то ест"},
    {ORTH: "т.н.", NORM: "таканаречен"},
    {ORTH: "бр.", NORM: "број"},
    {ORTH: "гр.", NORM: "град"},
    {ORTH: "др.", NORM: "другар"},
    {ORTH: "и др.", NORM: "и друго"},
    {ORTH: "и сл.", NORM: "и слично"},
    {ORTH: "кн.", NORM: "книга"},
    {ORTH: "мн.", NORM: "множина"},
    {ORTH: "на пр.", NORM: "на пример"},
    {ORTH: "св.", NORM: "свети"},
    {ORTH: "сп.", NORM: "списание"},
    {ORTH: "с.", NORM: "страница"},
    {ORTH: "стр.", NORM: "страница"},
    {ORTH: "чл.", NORM: "член"},
    {ORTH: "арх.", NORM: "архитект"},
    {ORTH: "бел.", NORM: "белешка"},
    {ORTH: "гимн.", NORM: "гимназија"},
    {ORTH: "ден.", NORM: "денар"},
    {ORTH: "ул.", NORM: "улица"},
    {ORTH: "инж.", NORM: "инженер"},
    {ORTH: "проф.", NORM: "професор"},
    {ORTH: "студ.", NORM: "студент"},
    {ORTH: "бот.", NORM: "ботаника"},
    {ORTH: "мат.", NORM: "математика"},
    {ORTH: "мед.", NORM: "медицина"},
    {ORTH: "прил.", NORM: "прилог"},
    {ORTH: "прид.", NORM: "придавка"},
    {ORTH: "сврз.", NORM: "сврзник"},
    {ORTH: "физ.", NORM: "физика"},
    {ORTH: "хем.", NORM: "хемија"},
    {ORTH: "пр. н.", NORM: "природни науки"},
    {ORTH: "истор.", NORM: "историја"},
    {ORTH: "геогр.", NORM: "географија"},
    {ORTH: "литер.", NORM: "литература"},
]

for abbr in _abbr_dot_exc:
    _exc[abbr[ORTH]] = [abbr]


TOKENIZER_EXCEPTIONS = _exc



STOP_WORDS = set(
    """
alle allerede alt and andre annen annet at av

bak bare bedre beste blant ble bli blir blitt bris by både

da dag de del dem den denne der dermed det dette disse du

eller en enn er et ett etter

fem fikk fire fjor flere folk for fortsatt fra fram
funnet få får fått før først første

gang gi gikk gjennom gjorde gjort gjør gjøre god godt grunn gå går

ha hadde ham han hans har hele helt henne hennes her hun

i ifølge igjen ikke ingen inn

ja jeg

kamp kampen kan kl klart kom komme kommer kontakt kort kroner kunne kveld

la laget land landet langt leder ligger like litt løpet

man mange med meg mellom men mener mennesker mens mer mot mye må mål måtte

ned neste noe noen nok ny nye nå når

og også om opp opplyser oss over

personer plass poeng på

runde rundt

sa saken samme sammen samtidig satt se seg seks selv senere ser sett
siden sier sin sine siste sitt skal skriver skulle slik som sted stedet stor
store står svært så

ta tatt tid tidligere til tilbake tillegg tok tror

under ut uten utenfor

vant var ved veldig vi videre viktig vil ville viser vår være vært

å år

ønsker
""".split()
)



import pytest
from spacy.lang.mk.lex_attrs import like_num


def test_tokenizer_handles_long_text(mk_tokenizer):
    text = """
    Во организациските работи или на нашите собранија со членството, никој од нас не зборуваше за
    организацијата и идеологијата. Работна беше нашата работа, а не идеолошка. Што се однесува до социјализмот на
    Делчев, неговата дејност зборува сама за себе - спротивно. Во суштина, водачите си имаа свои основни погледи и
    свои разбирања за положбата и работите, коишто стоеја пред нив и ги завршуваа со голема упорност, настојчивост и
    насоченост. Значи, идеологија имаше, само што нивната идеологија имаше своја оригиналност. Македонија денеска,
    чиста рожба на животот и положбата во Македонија, кои му служеа како база на неговите побуди, беше дејност која
    имаше потреба од ум за да си најде своја смисла. Таквата идеологија и заемното дејство на умот и срцето му
    помогнаа на Делчев да не се занесе по патот на својата идеологија... Во суштина, Организацијата и нејзините
    водачи имаа свои разбирања за работите и положбата во идеен поглед, но тоа беше врската, животот и положбата во
    Македонија и го внесуваа во својата идеологија гласот на своето срце, и на крај, прибегнуваа до умот,
    за да најдат смисла или да ѝ дадат. Тоа содејство и заемен сооднос на умот и срцето му помогнаа на Делчев да ја
    држи својата идеологија во сообразност со положбата на работите... Водачите навистина направија една жртва
    бидејќи на населението не му зборуваа за своите мисли и идеи. Тие се одрекоа од секаква субјективност во своите
    мисли. Целта беше да не се зголемуваат целите и задачите како и преданоста во работата. Населението не можеше да
    ги разбере овие идеи...
    """
    tokens = mk_tokenizer(text)
    assert len(tokens) == 297


@pytest.mark.parametrize(
    "word,match",
    [
        ("10", True),
        ("1", True),
        ("10.000", True),
        ("1000", True),
        ("бројка", False),
        ("999,0", True),
        ("еден", True),
        ("два", True),
        ("цифра", False),
        ("десет", True),
        ("сто", True),
        ("број", False),
        ("илјада", True),
        ("илјади", True),
        ("милион", True),
        (",", False),
        ("милијарда", True),
        ("билион", True),
    ],
)
def test_mk_lex_attrs_like_number(mk_tokenizer, word, match):
    tokens = mk_tokenizer(word)
    assert len(tokens) == 1
    assert tokens[0].like_num == match


@pytest.mark.parametrize("word", ["двесте", "два-три", "пет-шест"])
def test_mk_lex_attrs_capitals(word):
    assert like_num(word)
    assert like_num(word.upper())


@pytest.mark.parametrize(
    "word",
    [
        "првиот",
        "втора",
        "четврт",
        "четвртата",
        "петти",
        "петто",
        "стоти",
        "шеесетите",
        "седумдесетите",
    ],
)
def test_mk_lex_attrs_like_number_for_ordinal(word):
    assert like_num(word)



from typing import Optional, Callable
from thinc.api import Model
from .lemmatizer import MacedonianLemmatizer
from .stop_words import STOP_WORDS
from .tokenizer_exceptions import TOKENIZER_EXCEPTIONS
from .lex_attrs import LEX_ATTRS
from ..tokenizer_exceptions import BASE_EXCEPTIONS

from ...language import Language, BaseDefaults
from ...attrs import LANG
from ...util import update_exc
from ...lookups import Lookups


class MacedonianDefaults(BaseDefaults):
    lex_attr_getters = dict(Language.Defaults.lex_attr_getters)
    lex_attr_getters[LANG] = lambda text: "mk"

    # Optional: replace flags with custom functions, e.g. like_num()
    lex_attr_getters.update(LEX_ATTRS)

    # Merge base exceptions and custom tokenizer exceptions
    tokenizer_exceptions = update_exc(BASE_EXCEPTIONS, TOKENIZER_EXCEPTIONS)
    stop_words = STOP_WORDS

    @classmethod
    def create_lemmatizer(cls, nlp=None, lookups=None):
        if lookups is None:
            lookups = Lookups()
        return MacedonianLemmatizer(lookups)


class Macedonian(Language):
    lang = "mk"
    Defaults = MacedonianDefaults


@Macedonian.factory(
    "lemmatizer",
    assigns=["token.lemma"],
    default_config={
        "model": None,
        "mode": "rule",
        "overwrite": False,
        "scorer": {"@scorers": "spacy.lemmatizer_scorer.v1"},
    },
    default_score_weights={"lemma_acc": 1.0},
)
def make_lemmatizer(
    nlp: Language,
    model: Optional[Model],
    name: str,
    mode: str,
    overwrite: bool,
    scorer: Optional[Callable],
):
    return MacedonianLemmatizer(
        nlp.vocab, model, name, mode=mode, overwrite=overwrite, scorer=scorer
    )


__all__ = ["Macedonian"]



from typing import Optional, Iterable, Dict, Set, List, Any, Callable, Tuple
from typing import TYPE_CHECKING
import numpy as np
from collections import defaultdict

from .training import Example
from .tokens import Token, Doc, Span
from .errors import Errors
from .util import get_lang_class, SimpleFrozenList
from .morphology import Morphology

if TYPE_CHECKING:
    # This lets us add type hints for mypy etc. without causing circular imports
    from .language import Language  # noqa: F401


DEFAULT_PIPELINE = ("senter", "tagger", "morphologizer", "parser", "ner", "textcat")
MISSING_VALUES = frozenset([None, 0, ""])


class PRFScore:
    """A precision / recall / F score."""

    def __init__(
        self,
        *,
        tp: int = 0,
        fp: int = 0,
        fn: int = 0,
    ) -> None:
        self.tp = tp
        self.fp = fp
        self.fn = fn

    def __len__(self) -> int:
        return self.tp + self.fp + self.fn

    def __iadd__(self, other):
        self.tp += other.tp
        self.fp += other.fp
        self.fn += other.fn
        return self

    def __add__(self, other):
        return PRFScore(
            tp=self.tp + other.tp, fp=self.fp + other.fp, fn=self.fn + other.fn
        )

    def score_set(self, cand: set, gold: set) -> None:
        self.tp += len(cand.intersection(gold))
        self.fp += len(cand - gold)
        self.fn += len(gold - cand)

    @property
    def precision(self) -> float:
        return self.tp / (self.tp + self.fp + 1e-100)

    @property
    def recall(self) -> float:
        return self.tp / (self.tp + self.fn + 1e-100)

    @property
    def fscore(self) -> float:
        p = self.precision
        r = self.recall
        return 2 * ((p * r) / (p + r + 1e-100))

    def to_dict(self) -> Dict[str, float]:
        return {"p": self.precision, "r": self.recall, "f": self.fscore}


class ROCAUCScore:
    """An AUC ROC score. This is only defined for binary classification.
    Use the method is_binary before calculating the score, otherwise it
    may throw an error."""

    def __init__(self) -> None:
        self.golds: List[Any] = []
        self.cands: List[Any] = []
        self.saved_score = 0.0
        self.saved_score_at_len = 0

    def score_set(self, cand, gold) -> None:
        self.cands.append(cand)
        self.golds.append(gold)

    def is_binary(self):
        return len(np.unique(self.golds)) == 2

    @property
    def score(self):
        if not self.is_binary():
            raise ValueError(Errors.E165.format(label=set(self.golds)))
        if len(self.golds) == self.saved_score_at_len:
            return self.saved_score
        self.saved_score = _roc_auc_score(self.golds, self.cands)
        self.saved_score_at_len = len(self.golds)
        return self.saved_score


class Scorer:
    """Compute evaluation scores."""

    def __init__(
        self,
        nlp: Optional["Language"] = None,
        default_lang: str = "xx",
        default_pipeline: Iterable[str] = DEFAULT_PIPELINE,
        **cfg,
    ) -> None:
        """Initialize the Scorer.

        DOCS: https://spacy.io/api/scorer#init
        """
        self.cfg = cfg
        if nlp:
            self.nlp = nlp
        else:
            nlp = get_lang_class(default_lang)()
            for pipe in default_pipeline:
                nlp.add_pipe(pipe)
            self.nlp = nlp

    def score(self, examples: Iterable[Example]) -> Dict[str, Any]:
        """Evaluate a list of Examples.

        examples (Iterable[Example]): The predicted annotations + correct annotations.
        RETURNS (Dict): A dictionary of scores.

        DOCS: https://spacy.io/api/scorer#score
        """
        scores = {}
        if hasattr(self.nlp.tokenizer, "score"):
            scores.update(self.nlp.tokenizer.score(examples, **self.cfg))  # type: ignore
        for name, component in self.nlp.pipeline:
            if hasattr(component, "score"):
                scores.update(component.score(examples, **self.cfg))
        return scores

    @staticmethod
    def score_tokenization(examples: Iterable[Example], **cfg) -> Dict[str, Any]:
        """Returns accuracy and PRF scores for tokenization.
        * token_acc: # correct tokens / # gold tokens
        * token_p/r/f: PRF for token character spans

        examples (Iterable[Example]): Examples to score
        RETURNS (Dict[str, Any]): A dictionary containing the scores
            token_acc/p/r/f.

        DOCS: https://spacy.io/api/scorer#score_tokenization
        """
        acc_score = PRFScore()
        prf_score = PRFScore()
        for example in examples:
            gold_doc = example.reference
            pred_doc = example.predicted
            if gold_doc.has_unknown_spaces:
                continue
            align = example.alignment
            gold_spans = set()
            pred_spans = set()
            for token in gold_doc:
                if token.orth_.isspace():
                    continue
                gold_spans.add((token.idx, token.idx + len(token)))
            for token in pred_doc:
                if token.orth_.isspace():
                    continue
                pred_spans.add((token.idx, token.idx + len(token)))
                if align.x2y.lengths[token.i] != 1:
                    acc_score.fp += 1
                else:
                    acc_score.tp += 1
            prf_score.score_set(pred_spans, gold_spans)
        if len(acc_score) > 0:
            return {
                "token_acc": acc_score.fscore,
                "token_p": prf_score.precision,
                "token_r": prf_score.recall,
                "token_f": prf_score.fscore,
            }
        else:
            return {
                "token_acc": None,
                "token_p": None,
                "token_r": None,
                "token_f": None,
            }

    @staticmethod
    def score_token_attr(
        examples: Iterable[Example],
        attr: str,
        *,
        getter: Callable[[Token, str], Any] = getattr,
        missing_values: Set[Any] = MISSING_VALUES,  # type: ignore[assignment]
        **cfg,
    ) -> Dict[str, Any]:
        """Returns an accuracy score for a token-level attribute.

        examples (Iterable[Example]): Examples to score
        attr (str): The attribute to score.
        getter (Callable[[Token, str], Any]): Defaults to getattr. If provided,
            getter(token, attr) should return the value of the attribute for an
            individual token.
        missing_values (Set[Any]): Attribute values to treat as missing annotation
            in the reference annotation.
        RETURNS (Dict[str, Any]): A dictionary containing the accuracy score
            under the key attr_acc.

        DOCS: https://spacy.io/api/scorer#score_token_attr
        """
        tag_score = PRFScore()
        for example in examples:
            gold_doc = example.reference
            pred_doc = example.predicted
            align = example.alignment
            gold_tags = set()
            missing_indices = set()
            for gold_i, token in enumerate(gold_doc):
                value = getter(token, attr)
                if value not in missing_values:
                    gold_tags.add((gold_i, getter(token, attr)))
                else:
                    missing_indices.add(gold_i)
            pred_tags = set()
            for token in pred_doc:
                if token.orth_.isspace():
                    continue
                if align.x2y.lengths[token.i] == 1:
                    gold_i = align.x2y[token.i][0]
                    if gold_i not in missing_indices:
                        pred_tags.add((gold_i, getter(token, attr)))
            tag_score.score_set(pred_tags, gold_tags)
        score_key = f"{attr}_acc"
        if len(tag_score) == 0:
            return {score_key: None}
        else:
            return {score_key: tag_score.fscore}

    @staticmethod
    def score_token_attr_per_feat(
        examples: Iterable[Example],
        attr: str,
        *,
        getter: Callable[[Token, str], Any] = getattr,
        missing_values: Set[Any] = MISSING_VALUES,  # type: ignore[assignment]
        **cfg,
    ) -> Dict[str, Any]:
        """Return micro PRF and PRF scores per feat for a token attribute in
        UFEATS format.

        examples (Iterable[Example]): Examples to score
        attr (str): The attribute to score.
        getter (Callable[[Token, str], Any]): Defaults to getattr. If provided,
            getter(token, attr) should return the value of the attribute for an
            individual token.
        missing_values (Set[Any]): Attribute values to treat as missing
            annotation in the reference annotation.
        RETURNS (dict): A dictionary containing the micro PRF scores under the
            key attr_micro_p/r/f and the per-feat PRF scores under
            attr_per_feat.
        """
        micro_score = PRFScore()
        per_feat = {}
        for example in examples:
            pred_doc = example.predicted
            gold_doc = example.reference
            align = example.alignment
            gold_per_feat: Dict[str, Set] = {}
            missing_indices = set()
            for gold_i, token in enumerate(gold_doc):
                value = getter(token, attr)
                morph = gold_doc.vocab.strings[value]
                if value not in missing_values and morph != Morphology.EMPTY_MORPH:
                    for feat in morph.split(Morphology.FEATURE_SEP):
                        field, values = feat.split(Morphology.FIELD_SEP)
                        if field not in per_feat:
                            per_feat[field] = PRFScore()
                        if field not in gold_per_feat:
                            gold_per_feat[field] = set()
                        gold_per_feat[field].add((gold_i, feat))
                else:
                    missing_indices.add(gold_i)
            pred_per_feat: Dict[str, Set] = {}
            for token in pred_doc:
                if token.orth_.isspace():
                    continue
                if align.x2y.lengths[token.i] == 1:
                    gold_i = align.x2y[token.i][0]
                    if gold_i not in missing_indices:
                        value = getter(token, attr)
                        morph = gold_doc.vocab.strings[value]
                        if (
                            value not in missing_values
                            and morph != Morphology.EMPTY_MORPH
                        ):
                            for feat in morph.split(Morphology.FEATURE_SEP):
                                field, values = feat.split(Morphology.FIELD_SEP)
                                if field not in per_feat:
                                    per_feat[field] = PRFScore()
                                if field not in pred_per_feat:
                                    pred_per_feat[field] = set()
                                pred_per_feat[field].add((gold_i, feat))
            for field in per_feat:
                micro_score.score_set(
                    pred_per_feat.get(field, set()), gold_per_feat.get(field, set())
                )
                per_feat[field].score_set(
                    pred_per_feat.get(field, set()), gold_per_feat.get(field, set())
                )
        result: Dict[str, Any] = {}
        if len(micro_score) > 0:
            result[f"{attr}_micro_p"] = micro_score.precision
            result[f"{attr}_micro_r"] = micro_score.recall
            result[f"{attr}_micro_f"] = micro_score.fscore
            result[f"{attr}_per_feat"] = {k: v.to_dict() for k, v in per_feat.items()}
        else:
            result[f"{attr}_micro_p"] = None
            result[f"{attr}_micro_r"] = None
            result[f"{attr}_micro_f"] = None
            result[f"{attr}_per_feat"] = None
        return result

    @staticmethod
    def score_spans(
        examples: Iterable[Example],
        attr: str,
        *,
        getter: Callable[[Doc, str], Iterable[Span]] = getattr,
        has_annotation: Optional[Callable[[Doc], bool]] = None,
        labeled: bool = True,
        allow_overlap: bool = False,
        **cfg,
    ) -> Dict[str, Any]:
        """Returns PRF scores for labeled spans.

        examples (Iterable[Example]): Examples to score
        attr (str): The attribute to score.
        getter (Callable[[Doc, str], Iterable[Span]]): Defaults to getattr. If
            provided, getter(doc, attr) should return the spans for the
            individual doc.
        has_annotation (Optional[Callable[[Doc], bool]]) should return whether a `Doc`
            has annotation for this `attr`. Docs without annotation are skipped for
            scoring purposes.
        labeled (bool): Whether or not to include label information in
            the evaluation. If set to 'False', two spans will be considered
            equal if their start and end match, irrespective of their label.
        allow_overlap (bool): Whether or not to allow overlapping spans.
            If set to 'False', the alignment will automatically resolve conflicts.
        RETURNS (Dict[str, Any]): A dictionary containing the PRF scores under
            the keys attr_p/r/f and the per-type PRF scores under attr_per_type.

        DOCS: https://spacy.io/api/scorer#score_spans
        """
        score = PRFScore()
        score_per_type = dict()
        for example in examples:
            pred_doc = example.predicted
            gold_doc = example.reference
            # Option to handle docs without annotation for this attribute
            if has_annotation is not None and not has_annotation(gold_doc):
                continue
            # Find all labels in gold
            labels = set([k.label_ for k in getter(gold_doc, attr)])
            # If labeled, find all labels in pred
            if has_annotation is None or (
                has_annotation is not None and has_annotation(pred_doc)
            ):
                labels |= set([k.label_ for k in getter(pred_doc, attr)])
            # Set up all labels for per type scoring and prepare gold per type
            gold_per_type: Dict[str, Set] = {label: set() for label in labels}
            for label in labels:
                if label not in score_per_type:
                    score_per_type[label] = PRFScore()
            # Find all predidate labels, for all and per type
            gold_spans = set()
            pred_spans = set()
            for span in getter(gold_doc, attr):
                gold_span: Tuple
                if labeled:
                    gold_span = (span.label_, span.start, span.end - 1)
                else:
                    gold_span = (span.start, span.end - 1)
                gold_spans.add(gold_span)
                gold_per_type[span.label_].add(gold_span)
            pred_per_type: Dict[str, Set] = {label: set() for label in labels}
            if has_annotation is None or (
                has_annotation is not None and has_annotation(pred_doc)
            ):
                for span in example.get_aligned_spans_x2y(
                    getter(pred_doc, attr), allow_overlap
                ):
                    pred_span: Tuple
                    if labeled:
                        pred_span = (span.label_, span.start, span.end - 1)
                    else:
                        pred_span = (span.start, span.end - 1)
                    pred_spans.add(pred_span)
                    pred_per_type[span.label_].add(pred_span)
            # Scores per label
            if labeled:
                for k, v in score_per_type.items():
                    if k in pred_per_type:
                        v.score_set(pred_per_type[k], gold_per_type[k])
            # Score for all labels
            score.score_set(pred_spans, gold_spans)
        # Assemble final result
        final_scores: Dict[str, Any] = {
            f"{attr}_p": None,
            f"{attr}_r": None,
            f"{attr}_f": None,
        }
        if labeled:
            final_scores[f"{attr}_per_type"] = None
        if len(score) > 0:
            final_scores[f"{attr}_p"] = score.precision
            final_scores[f"{attr}_r"] = score.recall
            final_scores[f"{attr}_f"] = score.fscore
            if labeled:
                final_scores[f"{attr}_per_type"] = {
                    k: v.to_dict() for k, v in score_per_type.items()
                }
        return final_scores

    @staticmethod
    def score_cats(
        examples: Iterable[Example],
        attr: str,
        *,
        getter: Callable[[Doc, str], Any] = getattr,
        labels: Iterable[str] = SimpleFrozenList(),
        multi_label: bool = True,
        positive_label: Optional[str] = None,
        threshold: Optional[float] = None,
        **cfg,
    ) -> Dict[str, Any]:
        """Returns PRF and ROC AUC scores for a doc-level attribute with a
        dict with scores for each label like Doc.cats. The reported overall
        score depends on the scorer settings.

        examples (Iterable[Example]): Examples to score
        attr (str): The attribute to score.
        getter (Callable[[Doc, str], Any]): Defaults to getattr. If provided,
            getter(doc, attr) should return the values for the individual doc.
        labels (Iterable[str]): The set of possible labels. Defaults to [].
        multi_label (bool): Whether the attribute allows multiple labels.
            Defaults to True. When set to False (exclusive labels), missing
            gold labels are interpreted as 0.0.
        positive_label (str): The positive label for a binary task with
            exclusive classes. Defaults to None.
        threshold (float): Cutoff to consider a prediction "positive". Defaults
            to 0.5 for multi-label, and 0.0 (i.e. whatever's highest scoring)
            otherwise.
        RETURNS (Dict[str, Any]): A dictionary containing the scores, with
            inapplicable scores as None:
            for all:
                attr_score (one of attr_micro_f / attr_macro_f / attr_macro_auc),
                attr_score_desc (text description of the overall score),
                attr_micro_p,
                attr_micro_r,
                attr_micro_f,
                attr_macro_p,
                attr_macro_r,
                attr_macro_f,
                attr_macro_auc,
                attr_f_per_type,
                attr_auc_per_type

        DOCS: https://spacy.io/api/scorer#score_cats
        """
        if threshold is None:
            threshold = 0.5 if multi_label else 0.0
        f_per_type = {label: PRFScore() for label in labels}
        auc_per_type = {label: ROCAUCScore() for label in labels}
        labels = set(labels)
        if labels:
            for eg in examples:
                labels.update(eg.predicted.cats.keys())
                labels.update(eg.reference.cats.keys())
        for example in examples:
            # Through this loop, None in the gold_cats indicates missing label.
            pred_cats = getter(example.predicted, attr)
            gold_cats = getter(example.reference, attr)

            for label in labels:
                pred_score = pred_cats.get(label, 0.0)
                gold_score = gold_cats.get(label)
                if not gold_score and not multi_label:
                    gold_score = 0.0
                if gold_score is not None:
                    auc_per_type[label].score_set(pred_score, gold_score)
            if multi_label:
                for label in labels:
                    pred_score = pred_cats.get(label, 0.0)
                    gold_score = gold_cats.get(label)
                    if gold_score is not None:
                        if pred_score >= threshold and gold_score > 0:
                            f_per_type[label].tp += 1
                        elif pred_score >= threshold and gold_score == 0:
                            f_per_type[label].fp += 1
                        elif pred_score < threshold and gold_score > 0:
                            f_per_type[label].fn += 1
            elif pred_cats and gold_cats:
                # Get the highest-scoring for each.
                pred_label, pred_score = max(pred_cats.items(), key=lambda it: it[1])
                gold_label, gold_score = max(gold_cats.items(), key=lambda it: it[1])
                if pred_label == gold_label and pred_score >= threshold:
                    f_per_type[pred_label].tp += 1
                else:
                    f_per_type[gold_label].fn += 1
                    if pred_score >= threshold:
                        f_per_type[pred_label].fp += 1
            elif gold_cats:
                gold_label, gold_score = max(gold_cats, key=lambda it: it[1])
                if gold_score > 0:
                    f_per_type[gold_label].fn += 1
            elif pred_cats:
                pred_label, pred_score = max(pred_cats.items(), key=lambda it: it[1])
                if pred_score >= threshold:
                    f_per_type[pred_label].fp += 1
        micro_prf = PRFScore()
        for label_prf in f_per_type.values():
            micro_prf.tp += label_prf.tp
            micro_prf.fn += label_prf.fn
            micro_prf.fp += label_prf.fp
        n_cats = len(f_per_type) + 1e-100
        macro_p = sum(prf.precision for prf in f_per_type.values()) / n_cats
        macro_r = sum(prf.recall for prf in f_per_type.values()) / n_cats
        macro_f = sum(prf.fscore for prf in f_per_type.values()) / n_cats
        # Limit macro_auc to those labels with gold annotations,
        # but still divide by all cats to avoid artificial boosting of datasets with missing labels
        macro_auc = (
            sum(auc.score if auc.is_binary() else 0.0 for auc in auc_per_type.values())
            / n_cats
        )
        results: Dict[str, Any] = {
            f"{attr}_score": None,
            f"{attr}_score_desc": None,
            f"{attr}_micro_p": micro_prf.precision,
            f"{attr}_micro_r": micro_prf.recall,
            f"{attr}_micro_f": micro_prf.fscore,
            f"{attr}_macro_p": macro_p,
            f"{attr}_macro_r": macro_r,
            f"{attr}_macro_f": macro_f,
            f"{attr}_macro_auc": macro_auc,
            f"{attr}_f_per_type": {k: v.to_dict() for k, v in f_per_type.items()},
            f"{attr}_auc_per_type": {
                k: v.score if v.is_binary() else None for k, v in auc_per_type.items()
            },
        }
        if len(labels) == 2 and not multi_label and positive_label:
            positive_label_f = results[f"{attr}_f_per_type"][positive_label]["f"]
            results[f"{attr}_score"] = positive_label_f
            results[f"{attr}_score_desc"] = f"F ({positive_label})"
        elif not multi_label:
            results[f"{attr}_score"] = results[f"{attr}_macro_f"]
            results[f"{attr}_score_desc"] = "macro F"
        else:
            results[f"{attr}_score"] = results[f"{attr}_macro_auc"]
            results[f"{attr}_score_desc"] = "macro AUC"
        return results

    @staticmethod
    def score_links(
        examples: Iterable[Example], *, negative_labels: Iterable[str], **cfg
    ) -> Dict[str, Any]:
        """Returns PRF for predicted links on the entity level.
        To disentangle the performance of the NEL from the NER,
        this method only evaluates NEL links for entities that overlap
        between the gold reference and the predictions.

        examples (Iterable[Example]): Examples to score
        negative_labels (Iterable[str]): The string values that refer to no annotation (e.g. "NIL")
        RETURNS (Dict[str, Any]): A dictionary containing the scores.

        DOCS: https://spacy.io/api/scorer#score_links
        """
        f_per_type = {}
        for example in examples:
            gold_ent_by_offset = {}
            for gold_ent in example.reference.ents:
                gold_ent_by_offset[(gold_ent.start_char, gold_ent.end_char)] = gold_ent

            for pred_ent in example.predicted.ents:
                gold_span = gold_ent_by_offset.get(
                    (pred_ent.start_char, pred_ent.end_char), None
                )
                if gold_span is not None:
                    label = gold_span.label_
                    if label not in f_per_type:
                        f_per_type[label] = PRFScore()
                    gold = gold_span.kb_id_
                    # only evaluating entities that overlap between gold and pred,
                    # to disentangle the performance of the NEL from the NER
                    if gold is not None:
                        pred = pred_ent.kb_id_
                        if gold in negative_labels and pred in negative_labels:
                            # ignore true negatives
                            pass
                        elif gold == pred:
                            f_per_type[label].tp += 1
                        elif gold in negative_labels:
                            f_per_type[label].fp += 1
                        elif pred in negative_labels:
                            f_per_type[label].fn += 1
                        else:
                            # a wrong prediction (e.g. Q42 != Q3) counts as both a FP as well as a FN
                            f_per_type[label].fp += 1
                            f_per_type[label].fn += 1
        micro_prf = PRFScore()
        for label_prf in f_per_type.values():
            micro_prf.tp += label_prf.tp
            micro_prf.fn += label_prf.fn
            micro_prf.fp += label_prf.fp
        n_labels = len(f_per_type) + 1e-100
        macro_p = sum(prf.precision for prf in f_per_type.values()) / n_labels
        macro_r = sum(prf.recall for prf in f_per_type.values()) / n_labels
        macro_f = sum(prf.fscore for prf in f_per_type.values()) / n_labels
        results = {
            f"nel_score": micro_prf.fscore,
            f"nel_score_desc": "micro F",
            f"nel_micro_p": micro_prf.precision,
            f"nel_micro_r": micro_prf.recall,
            f"nel_micro_f": micro_prf.fscore,
            f"nel_macro_p": macro_p,
            f"nel_macro_r": macro_r,
            f"nel_macro_f": macro_f,
            f"nel_f_per_type": {k: v.to_dict() for k, v in f_per_type.items()},
        }
        return results

    @staticmethod
    def score_deps(
        examples: Iterable[Example],
        attr: str,
        *,
        getter: Callable[[Token, str], Any] = getattr,
        head_attr: str = "head",
        head_getter: Callable[[Token, str], Token] = getattr,
        ignore_labels: Iterable[str] = SimpleFrozenList(),
        missing_values: Set[Any] = MISSING_VALUES,  # type: ignore[assignment]
        **cfg,
    ) -> Dict[str, Any]:
        """Returns the UAS, LAS, and LAS per type scores for dependency
        parses.

        examples (Iterable[Example]): Examples to score
        attr (str): The attribute containing the dependency label.
        getter (Callable[[Token, str], Any]): Defaults to getattr. If provided,
            getter(token, attr) should return the value of the attribute for an
            individual token.
        head_attr (str): The attribute containing the head token. Defaults to
            'head'.
        head_getter (Callable[[Token, str], Token]): Defaults to getattr. If provided,
            head_getter(token, attr) should return the value of the head for an
            individual token.
        ignore_labels (Tuple): Labels to ignore while scoring (e.g., punct).
        missing_values (Set[Any]): Attribute values to treat as missing annotation
            in the reference annotation.
        RETURNS (Dict[str, Any]): A dictionary containing the scores:
            attr_uas, attr_las, and attr_las_per_type.

        DOCS: https://spacy.io/api/scorer#score_deps
        """
        unlabelled = PRFScore()
        labelled = PRFScore()
        labelled_per_dep = dict()
        missing_indices = set()
        for example in examples:
            gold_doc = example.reference
            pred_doc = example.predicted
            align = example.alignment
            gold_deps = set()
            gold_deps_per_dep: Dict[str, Set] = {}
            for gold_i, token in enumerate(gold_doc):
                dep = getter(token, attr)
                head = head_getter(token, head_attr)
                if dep not in missing_values:
                    if dep not in ignore_labels:
                        gold_deps.add((gold_i, head.i, dep))
                        if dep not in labelled_per_dep:
                            labelled_per_dep[dep] = PRFScore()
                        if dep not in gold_deps_per_dep:
                            gold_deps_per_dep[dep] = set()
                        gold_deps_per_dep[dep].add((gold_i, head.i, dep))
                else:
                    missing_indices.add(gold_i)
            pred_deps = set()
            pred_deps_per_dep: Dict[str, Set] = {}
            for token in pred_doc:
                if token.orth_.isspace():
                    continue
                if align.x2y.lengths[token.i] != 1:
                    gold_i = None  # type: ignore
                else:
                    gold_i = align.x2y[token.i][0]
                if gold_i not in missing_indices:
                    dep = getter(token, attr)
                    head = head_getter(token, head_attr)
                    if dep not in ignore_labels and token.orth_.strip():
                        if align.x2y.lengths[head.i] == 1:
                            gold_head = align.x2y[head.i][0]
                        else:
                            gold_head = None
                        # None is indistinct, so we can't just add it to the set
                        # Multiple (None, None) deps are possible
                        if gold_i is None or gold_head is None:
                            unlabelled.fp += 1
                            labelled.fp += 1
                        else:
                            pred_deps.add((gold_i, gold_head, dep))
                            if dep not in labelled_per_dep:
                                labelled_per_dep[dep] = PRFScore()
                            if dep not in pred_deps_per_dep:
                                pred_deps_per_dep[dep] = set()
                            pred_deps_per_dep[dep].add((gold_i, gold_head, dep))
            labelled.score_set(pred_deps, gold_deps)
            for dep in labelled_per_dep:
                labelled_per_dep[dep].score_set(
                    pred_deps_per_dep.get(dep, set()), gold_deps_per_dep.get(dep, set())
                )
            unlabelled.score_set(
                set(item[:2] for item in pred_deps), set(item[:2] for item in gold_deps)
            )
        if len(unlabelled) > 0:
            return {
                f"{attr}_uas": unlabelled.fscore,
                f"{attr}_las": labelled.fscore,
                f"{attr}_las_per_type": {
                    k: v.to_dict() for k, v in labelled_per_dep.items()
                },
            }
        else:
            return {
                f"{attr}_uas": None,
                f"{attr}_las": None,
                f"{attr}_las_per_type": None,
            }


def get_ner_prf(examples: Iterable[Example], **kwargs) -> Dict[str, Any]:
    """Compute micro-PRF and per-entity PRF scores for a sequence of examples."""
    score_per_type = defaultdict(PRFScore)
    for eg in examples:
        if not eg.y.has_annotation("ENT_IOB"):
            continue
        golds = {(e.label_, e.start, e.end) for e in eg.y.ents}
        align_x2y = eg.alignment.x2y
        for pred_ent in eg.x.ents:
            if pred_ent.label_ not in score_per_type:
                score_per_type[pred_ent.label_] = PRFScore()
            indices = align_x2y[pred_ent.start : pred_ent.end]
            if len(indices):
                g_span = eg.y[indices[0] : indices[-1] + 1]
                # Check we aren't missing annotation on this span. If so,
                # our prediction is neither right nor wrong, we just
                # ignore it.
                if all(token.ent_iob != 0 for token in g_span):
                    key = (pred_ent.label_, indices[0], indices[-1] + 1)
                    if key in golds:
                        score_per_type[pred_ent.label_].tp += 1
                        golds.remove(key)
                    else:
                        score_per_type[pred_ent.label_].fp += 1
        for label, start, end in golds:
            score_per_type[label].fn += 1
    totals = PRFScore()
    for prf in score_per_type.values():
        totals += prf
    if len(totals) > 0:
        return {
            "ents_p": totals.precision,
            "ents_r": totals.recall,
            "ents_f": totals.fscore,
            "ents_per_type": {k: v.to_dict() for k, v in score_per_type.items()},
        }
    else:
        return {
            "ents_p": None,
            "ents_r": None,
            "ents_f": None,
            "ents_per_type": None,
        }


# The following implementation of roc_auc_score() is adapted from
# scikit-learn, which is distributed under the New BSD License.
# Copyright (c) 2007–2019 The scikit-learn developers.
# See licenses/3rd_party_licenses.txt
def _roc_auc_score(y_true, y_score):
    """Compute Area Under the Receiver Operating Characteristic Curve (ROC AUC)
    from prediction scores.

    Note: this implementation is restricted to the binary classification task

    Parameters
    ----------
    y_true : array, shape = [n_samples] or [n_samples, n_classes]
        True binary labels or binary label indicators.
        The multiclass case expects shape = [n_samples] and labels
        with values in ``range(n_classes)``.

    y_score : array, shape = [n_samples] or [n_samples, n_classes]
        Target scores, can either be probability estimates of the positive
        class, confidence values, or non-thresholded measure of decisions
        (as returned by "decision_function" on some classifiers). For binary
        y_true, y_score is supposed to be the score of the class with greater
        label. The multiclass case expects shape = [n_samples, n_classes]
        where the scores correspond to probability estimates.

    Returns
    -------
    auc : float

    References
    ----------
    .. [1] `Wikipedia entry for the Receiver operating characteristic
            <https://en.wikipedia.org/wiki/Receiver_operating_characteristic>`_

    .. [2] Fawcett T. An introduction to ROC analysis[J]. Pattern Recognition
           Letters, 2006, 27(8):861-874.

    .. [3] `Analyzing a portion of the ROC curve. McClish, 1989
            <https://www.ncbi.nlm.nih.gov/pubmed/2668680>`_
    """
    if len(np.unique(y_true)) != 2:
        raise ValueError(Errors.E165.format(label=np.unique(y_true)))
    fpr, tpr, _ = _roc_curve(y_true, y_score)
    return _auc(fpr, tpr)


def _roc_curve(y_true, y_score):
    """Compute Receiver operating characteristic (ROC)

    Note: this implementation is restricted to the binary classification task.

    Parameters
    ----------

    y_true : array, shape = [n_samples]
        True binary labels. If labels are not either {-1, 1} or {0, 1}, then
        pos_label should be explicitly given.

    y_score : array, shape = [n_samples]
        Target scores, can either be probability estimates of the positive
        class, confidence values, or non-thresholded measure of decisions
        (as returned by "decision_function" on some classifiers).

    Returns
    -------
    fpr : array, shape = [>2]
        Increasing false positive rates such that element i is the false
        positive rate of predictions with score >= thresholds[i].

    tpr : array, shape = [>2]
        Increasing true positive rates such that element i is the true
        positive rate of predictions with score >= thresholds[i].

    thresholds : array, shape = [n_thresholds]
        Decreasing thresholds on the decision function used to compute
        fpr and tpr. `thresholds[0]` represents no instances being predicted
        and is arbitrarily set to `max(y_score) + 1`.

    Notes
    -----
    Since the thresholds are sorted from low to high values, they
    are reversed upon returning them to ensure they correspond to both ``fpr``
    and ``tpr``, which are sorted in reversed order during their calculation.

    References
    ----------
    .. [1] `Wikipedia entry for the Receiver operating characteristic
            <https://en.wikipedia.org/wiki/Receiver_operating_characteristic>`_

    .. [2] Fawcett T. An introduction to ROC analysis[J]. Pattern Recognition
           Letters, 2006, 27(8):861-874.
    """
    fps, tps, thresholds = _binary_clf_curve(y_true, y_score)

    # Add an extra threshold position
    # to make sure that the curve starts at (0, 0)
    tps = np.r_[0, tps]
    fps = np.r_[0, fps]
    thresholds = np.r_[thresholds[0] + 1, thresholds]

    if fps[-1] <= 0:
        fpr = np.repeat(np.nan, fps.shape)
    else:
        fpr = fps / fps[-1]

    if tps[-1] <= 0:
        tpr = np.repeat(np.nan, tps.shape)
    else:
        tpr = tps / tps[-1]

    return fpr, tpr, thresholds


def _binary_clf_curve(y_true, y_score):
    """Calculate true and false positives per binary classification threshold.

    Parameters
    ----------
    y_true : array, shape = [n_samples]
        True targets of binary classification

    y_score : array, shape = [n_samples]
        Estimated probabilities or decision function

    Returns
    -------
    fps : array, shape = [n_thresholds]
        A count of false positives, at index i being the number of negative
        samples assigned a score >= thresholds[i]. The total number of
        negative samples is equal to fps[-1] (thus true negatives are given by
        fps[-1] - fps).

    tps : array, shape = [n_thresholds <= len(np.unique(y_score))]
        An increasing count of true positives, at index i being the number
        of positive samples assigned a score >= thresholds[i]. The total
        number of positive samples is equal to tps[-1] (thus false negatives
        are given by tps[-1] - tps).

    thresholds : array, shape = [n_thresholds]
        Decreasing score values.
    """
    pos_label = 1.0

    y_true = np.ravel(y_true)
    y_score = np.ravel(y_score)

    # make y_true a boolean vector
    y_true = y_true == pos_label

    # sort scores and corresponding truth values
    desc_score_indices = np.argsort(y_score, kind="mergesort")[::-1]
    y_score = y_score[desc_score_indices]
    y_true = y_true[desc_score_indices]
    weight = 1.0

    # y_score typically has many tied values. Here we extract
    # the indices associated with the distinct values. We also
    # concatenate a value for the end of the curve.
    distinct_value_indices = np.where(np.diff(y_score))[0]
    threshold_idxs = np.r_[distinct_value_indices, y_true.size - 1]

    # accumulate the true positives with decreasing threshold
    tps = _stable_cumsum(y_true * weight)[threshold_idxs]
    fps = 1 + threshold_idxs - tps
    return fps, tps, y_score[threshold_idxs]


def _stable_cumsum(arr, axis=None, rtol=1e-05, atol=1e-08):
    """Use high precision for cumsum and check that final value matches sum

    Parameters
    ----------
    arr : array-like
        To be cumulatively summed as flat
    axis : int, optional
        Axis along which the cumulative sum is computed.
        The default (None) is to compute the cumsum over the flattened array.
    rtol : float
        Relative tolerance, see ``np.allclose``
    atol : float
        Absolute tolerance, see ``np.allclose``
    """
    out = np.cumsum(arr, axis=axis, dtype=np.float64)
    expected = np.sum(arr, axis=axis, dtype=np.float64)
    if not np.all(
        np.isclose(
            out.take(-1, axis=axis), expected, rtol=rtol, atol=atol, equal_nan=True
        )
    ):
        raise ValueError(Errors.E163)
    return out


def _auc(x, y):
    """Compute Area Under the Curve (AUC) using the trapezoidal rule

    This is a general function, given points on a curve.  For computing the
    area under the ROC-curve, see :func:`roc_auc_score`.

    Parameters
    ----------
    x : array, shape = [n]
        x coordinates. These must be either monotonic increasing or monotonic
        decreasing.
    y : array, shape = [n]
        y coordinates.

    Returns
    -------
    auc : float
    """
    x = np.ravel(x)
    y = np.ravel(y)

    direction = 1
    dx = np.diff(x)
    if np.any(dx < 0):
        if np.all(dx <= 0):
            direction = -1
        else:
            raise ValueError(Errors.E164.format(x=x))

    area = direction * np.trapz(y, x)
    if isinstance(area, np.memmap):
        # Reductions such as .sum used internally in np.trapz do not return a
        # scalar by default for numpy.memmap instances contrary to
        # regular numpy.ndarray instances.
        area = area.dtype.type(area)
    return area
